@@ -61,46 +61,49 @@ struct Resources_s
 struct Parse_Options_s
 {
 	int verbosity;         /* Level of detail to give about the computation 0 */
-	int use_sat_solver;    /* Use the Boolean SAT based parser */
-	int use_viterbi;       /* Use the Viterbi decoder-based parser */
+	Boolean use_sat_solver;/* Use the Boolean SAT based parser */
+	Boolean use_viterbi;   /* Use the Viterbi decoder-based parser */
 	int linkage_limit;     /* The maximum number of linkages processed 100 */
 	float disjunct_cost;   /* Max disjunct cost to allow */
-	int use_fat_links;     /* Look for fat linkages */
 	int min_null_count;    /* The minimum number of null links to allow */
 	int max_null_count;    /* The maximum number of null links to allow */
 	int null_block;        /* consecutive blocks of this many words are
 	                          considered as one null link  (default=1) */
-	int islands_ok;        /* If TRUE, then linkages with islands
+	Boolean islands_ok;    /* If TRUE, then linkages with islands
 	                          (separate component of the link graph)
 	                          will be generated (default=FALSE) */
 	int twopass_length;    /* min length for two-pass post processing */
 	int max_sentence_length;
 	int short_length;      /* Links that are limited in length can be
 	                        * no longer than this.  Default = 6 */
-	int all_short;         /* If true, there can be no connectors that are exempt */
-	int use_spell_guess;   /* Perform spell-guessing of unknown words. */ 
+	Boolean all_short;     /* If true, there can be no connectors that are exempt */
+	Boolean use_spell_guess; /* Perform spell-guessing of unknown words. */ 
 	Cost_Model cost_model; /* For sorting linkages in post_processing */
 	Resources resources;   /* For deciding when to abort the parsing */
 
 	/* Flags governing the command-line client; not used by parser */
-	int display_short;
-	int display_word_subscripts;  /* as in "dog.n" as opposed to "dog" */
-	int display_link_subscripts;  /* as in "Ss" as opposed to "S" */
-	int display_walls;
-	int display_union;            /* print squashed version of linkage with conjunction? */
-	int allow_null;               /* true if we allow null links in parsing */
-	int use_cluster_disjuncts;    /* if true, atttempt using a borader list of disjuncts */
-	int echo_on;                  /* true if we should echo the input sentence */
-	int batch_mode;               /* if true, process sentences non-interactively */
-	int panic_mode;               /* if true, parse in "panic mode" after all else fails */
-	int screen_width;             /* width of screen for displaying linkages */
-	int display_on;               /* if true, output graphical linkage diagram */
-	int display_postscript;       /* if true, output postscript linkage */
+	Boolean display_short;
+	Boolean display_word_subscripts;  /* as in "dog.n" as opposed to "dog" */
+	Boolean display_link_subscripts;  /* as in "Ss" as opposed to "S" */
+	Boolean display_walls;
+#ifdef USE_FAT_LINKAGES
+	Boolean use_fat_links;     /* Look for fat linkages */
+	Boolean display_union;     /* print squashed version of linkage with conjunction? */
+#endif /* USE_FAT_LINKAGES */
+	Boolean allow_null;        /* true if we allow null links in parsing */
+	Boolean use_cluster_disjuncts;    /* if true, atttempt using a borader list of disjuncts */
+	Boolean echo_on;           /* true if we should echo the input sentence */
+	Boolean batch_mode;        /* if true, process sentences non-interactively */
+	Boolean panic_mode;        /* if true, parse in "panic mode" after all else fails */
+	int screen_width;          /* width of screen for displaying linkages */
+	Boolean display_on;        /* if true, output graphical linkage diagram */
+	Boolean display_postscript;    /* if true, output postscript linkage */
 	ConstituentDisplayStyle display_constituents;     /* style for displaying constituent structure */
-	int display_bad;              /* if true, bad linkages are displayed */
-	int display_disjuncts;        /* if true, print disjuncts that were used */
-	int display_links;            /* if true, a list o' links is printed out */
-	int display_senses;           /* if true, sense candidates are printed out */
+	Boolean display_bad;       /* if true, bad linkages are displayed */
+	Boolean display_disjuncts; /* if true, print disjuncts that were used */
+	Boolean display_links;     /* if true, a list o' links is printed out */
+	Boolean display_senses;    /* if true, sense candidates are printed out */
+	Boolean display_suffixes;
 };
 
 struct Connector_set_s
@@ -121,7 +124,7 @@ struct Dictionary_s
 	Boolean         unknown_word_defined;
 	Boolean         left_wall_defined;
 	Boolean         right_wall_defined;
-	Boolean         have_empty_suffix;
+	Boolean         empty_word_defined;
 
 	/* Affixes are used during the tokenization stage. */
 	Dictionary      affix_table;
@@ -144,7 +147,9 @@ struct Dictionary_s
 
 	Postprocessor * postprocessor;
 	Postprocessor * constituent_pp;
+#ifdef USE_FAT_LINKAGES
 	Connector_set * andable_connector_set;  /* NULL=everything is andable */
+#endif /* USE_FAT_LINKAGES */
 	Connector_set * unlimited_connector_set; /* NULL=everthing is unlimited */
 	int             max_cost;
 	String_set *    string_set;  /* Set of link names constructed during parsing */
@@ -177,6 +182,7 @@ struct Label_node_s
 
 #define HT_SIZE (1<<10)
 
+#ifdef USE_FAT_LINKAGES
 struct And_data_s
 {
 	int          LT_bound;
@@ -188,6 +194,7 @@ struct And_data_s
 	int STAT_N_disjuncts;
 	int STAT_calls_to_equality_test;
 };
+#endif /* USE_FAT_LINKAGES */
 
 struct Parse_info_struct
 {
@@ -200,6 +207,7 @@ struct Parse_info_struct
 	int            N_links;
 	Link          link_array[MAX_LINKS];
 
+#ifdef USE_FAT_LINKAGES
 	/* Points to the image structure for each word.
 	 * NULL if not a fat word. */
 	Image_node ** image_array;
@@ -207,6 +215,7 @@ struct Parse_info_struct
 	/* Array of boolean flags, one per word. Set to TRUE if this 
 	 * word has a fat down link. FALSE otherise */
 	Boolean *has_fat_down;
+#endif /* USE_FAT_LINKAGES */
 
 	/* thread-safe random number state */
 	unsigned int rand_state;
@@ -232,7 +241,6 @@ struct Sentence_s
 	                               This is not the same as num alloced
 	                               because some may be non-canonical. */
 	int    num_valid_linkages;  /* number with no pp violations */
-	int    num_thin_linkages;   /* valid linkages which are not fat */
 	int    null_links;          /* null links allowed */
 	int    null_count;          /* number of null links in linkages */
 	Parse_info     parse_info;  /* set of parses for the sentence */
@@ -248,6 +256,7 @@ struct Sentence_s
 	match_context_t * match_ctxt; /* private state info used for matching */
 
 #ifdef USE_FAT_LINKAGES
+	int    num_thin_linkages;   /* valid linkages which are not fat */
 	/* Obsolete fat-linkage state */
 	char * is_conjunction;      /* Array of flags, one per word; set to
 	                               TRUE if conjunction, as defined by dictionary */
@@ -257,14 +266,16 @@ struct Sentence_s
 	And_data       and_data;    /* used to keep track of fat disjuncts */
 #endif /* USE_FAT_LINKAGES */
 
-	/* Post-processor priavte/internal state */
+	/* Post-processor private/internal state */
 	Boolean  q_pruned_rules;    /* don't prune rules more than once in p.p. */
 
 	/* thread-safe random number state */
 	unsigned int rand_state;
 
+#ifdef USE_SAT_SOLVER
 	/* Hook for the SAT solver */
 	void *hook;
+#endif /* USE_SAT_SOLVER */
 };
 
 /*********************************************************
@@ -345,20 +356,31 @@ struct Sublinkage_s
 	PP_data   pp_data;
 };
 
+#ifdef USE_FAT_LINKAGES
 typedef struct DIS_node_struct DIS_node;
+#endif /* USE_FAT_LINKAGES */
 
 struct Linkage_s
 {
 	int             num_words;  /* number of (tokenized) words */
 	const char *  * word;       /* array of word spellings */
 	Linkage_info*   info;       /* index and cost information */
-	int             num_sublinkages; /* One for thin linkages, bigger for fat */
-	int             current;    /* Allows user to select particular sublinkage */
+#ifdef USE_FAT_LINKAGES
 	Sublinkage *    sublinkage; /* A parse with conjunctions will have several */
-	int             unionized;  /* if TRUE, union of links has been computed */
+#else
+	Sublinkage      sublinkage; /* A parse with conjunctions will have several */
+#endif /* USE_FAT_LINKAGES */
+
 	Sentence        sent;
 	Parse_Options   opts;
+
+#ifdef USE_FAT_LINKAGES
+	int             num_sublinkages; /* One for thin linkages, bigger for fat */
+	int             current;    /* Allows user to select particular sublinkage */
+	int             unionized;  /* if TRUE, union of links has been computed */
 	DIS_node      * dis_con_tree; /* Disjunction-conjunction tree */
+#endif /* USE_FAT_LINKAGES */
+
 };
 
 
