@@ -1,7 +1,7 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
-/* Copyright 2008, 2009 Linas Vepstas                                    */
+/* Copyright 2008, 2009, 2013 Linas Vepstas                              */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
@@ -42,15 +42,18 @@ static int VDAL_compare_parse(Linkage_info * p1, Linkage_info * p2)
 	else if (p1->unused_word_cost != p2->unused_word_cost) {
 		return (p1->unused_word_cost - p2->unused_word_cost);
 	}
+#ifdef USE_FAT_LINKAGES
 	else if (p1->fat != p2->fat) {
 		return (p1->fat - p2->fat);
 	}
-	else if (p1->disjunct_cost != p2->disjunct_cost) {
-		return (p1->disjunct_cost - p2->disjunct_cost);
-	}
+#endif /* USE_FAT_LINKAGES */
+	else if (p1->disjunct_cost > p2->disjunct_cost) return 1;
+	else if (p1->disjunct_cost < p2->disjunct_cost) return -1;
+#ifdef USE_FAT_LINKAGES
 	else if (p1->and_cost != p2->and_cost) {
 		return (p1->and_cost - p2->and_cost);
 	}
+#endif /* USE_FAT_LINKAGES */
 	else {
 		return (p1->link_cost - p2->link_cost);
 	}
@@ -81,13 +84,13 @@ Parse_Options parse_options_create(void)
 	po->verbosity	 = 1;
 	po->linkage_limit = 100;
 	po->disjunct_cost = MAX_DISJUNCT_COST;
-	po->use_fat_links = FALSE;
 	po->min_null_count = 0;
 	po->max_null_count = 0;
 	po->null_block = 1;
 	po->islands_ok = FALSE;
 	po->use_spell_guess = TRUE;
 	po->use_sat_solver = FALSE;
+	po->use_viterbi = FALSE;
 
 #ifdef XXX_USE_CORPUS
 	/* Use the corpus cost model, if available.
@@ -109,7 +112,10 @@ Parse_Options parse_options_create(void)
 	po->display_word_subscripts = TRUE;
 	po->display_link_subscripts = TRUE;
 	po->display_walls = FALSE;
+#ifdef USE_FAT_LINKAGES
+	po->use_fat_links = FALSE;
 	po->display_union = FALSE;
+#endif /* USE_FAT_LINKAGES */
 	po->allow_null = TRUE;
 	po->use_cluster_disjuncts = FALSE;
 	po->echo_on = FALSE;
@@ -123,6 +129,7 @@ Parse_Options parse_options_create(void)
 	po->display_disjuncts = FALSE;
 	po->display_links = FALSE;
 	po->display_senses = FALSE;
+	po->display_suffixes = FALSE;
 
 	return po;
 }
@@ -191,12 +198,14 @@ int parse_options_get_use_viterbi(Parse_Options opts) {
 	return opts->use_viterbi;
 }
 
+#ifdef USE_FAT_LINKAGES
 void parse_options_set_use_fat_links(Parse_Options opts, int dummy) {
 	opts->use_fat_links = dummy;
 }
 int parse_options_get_use_fat_links(Parse_Options opts) {
 	return opts->use_fat_links;
 }
+#endif /* USE_FAT_LINKAGES */
 
 void parse_options_set_linkage_limit(Parse_Options opts, int dummy) {
 	opts->linkage_limit = dummy;
@@ -212,7 +221,7 @@ void parse_options_set_disjunct_costf(Parse_Options opts, float dummy) {
 	opts->disjunct_cost = dummy;
 }
 int parse_options_get_disjunct_cost(Parse_Options opts) {
-	return opts->disjunct_cost;
+	return (int) ceilf(opts->disjunct_cost);
 }
 float parse_options_get_disjunct_costf(Parse_Options opts) {
 	return opts->disjunct_cost;
@@ -413,6 +422,14 @@ int parse_options_get_display_senses(Parse_Options opts) {
 	return opts->display_senses;
 }
 
+int parse_options_get_display_suffixes(Parse_Options opts) {
+	return opts->display_suffixes;
+}
+
+void parse_options_set_display_suffixes(Parse_Options opts, int dummy) {
+	opts->display_suffixes = dummy;
+}
+
 void parse_options_set_display_walls(Parse_Options opts, int dummy) {
 	opts->display_walls = dummy;
 }
@@ -422,11 +439,17 @@ int parse_options_get_display_walls(Parse_Options opts) {
 }
 
 int parse_options_get_display_union(Parse_Options opts) {
+#ifdef USE_FAT_LINKAGES
 	return opts->display_union;
+#else
+	return 0;
+#endif /* USE_FAT_LINKAGES */
 }
 
 void parse_options_set_display_union(Parse_Options opts, int dummy) {
+#ifdef USE_FAT_LINKAGES
 	opts->display_union = dummy;
+#endif /* USE_FAT_LINKAGES */
 }
 
 int parse_options_timer_expired(Parse_Options opts) {
@@ -480,6 +503,7 @@ static void linkage_info_delete(Linkage_info *link_info, int sz)
 	xfree(link_info, sz);
 }
 
+#ifdef USE_FAT_LINKAGES
 static void free_andlists(Sentence sent)
 {
 	int L;
@@ -497,12 +521,15 @@ static void free_andlists(Sentence sent)
 	}
 	/* printf("\n"); */
 }
+#endif /* USE_FAT_LINKAGES */
 
 static void free_post_processing(Sentence sent)
 {
 	if (sent->link_info != NULL) {
 		/* postprocessing must have been done */
+#ifdef USE_FAT_LINKAGES
 		free_andlists(sent);
+#endif /* USE_FAT_LINKAGES */
 		linkage_info_delete(sent->link_info, sent->num_linkages_alloced);
 		sent->link_info = NULL;
 	}
@@ -514,10 +541,10 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 	int in, block_bottom, block_top;
 	int N_linkages_found, N_linkages_alloced;
 	int N_linkages_post_processed, N_valid_linkages;
-	int N_thin_linkages;
 	int overflowed;
 	Linkage_info *link_info;
 #ifdef USE_FAT_LINKAGES
+	int N_thin_linkages;
 	int only_canonical_allowed;
 	int canonical;
 #endif /* USE_FAT_LINKAGES */
@@ -542,7 +569,9 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 		sent->num_linkages_alloced = 0;
 		sent->num_linkages_post_processed = 0;
 		sent->num_valid_linkages = 0;
+#ifdef USE_FAT_LINKAGES
 		sent->num_thin_linkages = 0;
+#endif /* USE_FAT_LINKAGES */
 		sent->link_info = NULL;
 		return;
 	}
@@ -627,13 +656,17 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 
 	/* second pass: actually perform post-processing */
 	N_linkages_post_processed = 0;
+#ifdef USE_FAT_LINKAGES
 	N_thin_linkages = 0;
+#endif /* USE_FAT_LINKAGES */
 	for (in=0; (in < N_linkages_alloced) &&
 			   (!resources_exhausted(opts->resources)); in++)
 	{
 		Linkage_info *lifo = &link_info[N_linkages_post_processed];
 		extract_links(indices[in], sent->null_count, sent->parse_info);
 #ifdef USE_FAT_LINKAGES
+		lifo->fat = FALSE;
+		lifo->canonical = TRUE;
 		if (set_has_fat_down(sent))
 		{
 			canonical = is_canonical_linkage(sent);
@@ -646,13 +679,14 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 #endif /* USE_FAT_LINKAGES */
 		{
 			*lifo = analyze_thin_linkage(sent, opts, PP_SECOND_PASS);
-			lifo->fat = FALSE;
-			lifo->canonical = TRUE;
 		}
 		if (0 == lifo->N_violations)
 		{
 			N_valid_linkages++;
-			if (FALSE == lifo->fat) N_thin_linkages++;
+#ifdef USE_FAT_LINKAGES
+			if (FALSE == lifo->fat)
+				N_thin_linkages++;
+#endif /* USE_FAT_LINKAGES */
 		}
 		lifo->index = indices[in];
 		lg_corpus_score(sent, lifo);
@@ -700,7 +734,9 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 	sent->num_linkages_alloced = N_linkages_alloced;
 	sent->num_linkages_post_processed = N_linkages_post_processed;
 	sent->num_valid_linkages = N_valid_linkages;
+#ifdef USE_FAT_LINKAGES
 	sent->num_thin_linkages = N_thin_linkages;
+#endif /* USE_FAT_LINKAGES */
 	sent->link_info = link_info;
 
 	xfree(indices, N_linkages_alloced * sizeof(int));
@@ -741,6 +777,10 @@ Sentence sentence_create(const char *input_string, Dictionary dict)
 	sent->effective_dist = NULL;
 #endif /* USE_FAT_LINKAGES */
 
+#ifdef USE_SAT_SOLVER
+	sent->hook = NULL;
+#endif /* USE_SAT_SOLVER */
+
 	/* Make a copy of the input */
 	sent->orig_sentence = string_set_add (input_string, sent->string_set);
 
@@ -801,6 +841,8 @@ int sentence_split(Sentence sent, Parse_Options opts)
 	initialize_conjunction_tables(sent);
 #endif /* USE_FAT_LINKAGES */
 
+	/* If unknown_word is not defined, then no special processing
+	 * will be done for e.g. capitalized words. */
 	if (!(dict->unknown_word_defined && dict->use_unknown_word))
 	{
 		if (!sentence_in_dictionary(sent)) {
@@ -825,7 +867,7 @@ static void free_sentence_words(Sentence sent)
 		free_disjuncts(sent->word[i].d);
 		free(sent->word[i].alternatives);
 	}
-	free(sent->word);
+	free((void *) sent->word);
 	sent->word = NULL;
 	free(sent->post_quote);
 }
@@ -891,7 +933,11 @@ int sentence_null_count(Sentence sent) {
 
 int sentence_num_thin_linkages(Sentence sent) {
 	if (!sent) return 0;
+#ifdef USE_FAT_LINKAGES
 	return sent->num_thin_linkages;
+#else
+	return sent->num_valid_linkages;
+#endif /* USE_FAT_LINKAGES */
 }
 
 int sentence_num_linkages_found(Sentence sent) {
@@ -918,11 +964,15 @@ int sentence_num_violations(Sentence sent, int i) {
 }
 
 int sentence_and_cost(Sentence sent, int i) {
+#ifdef USE_FAT_LINKAGES
 	if (!sent) return 0;
 
 	/* The sat solver (currently) fails to fill in link_info */
 	if (!sent->link_info) return 0;
 	return sent->link_info[i].and_cost;
+#else
+	return 0;
+#endif /* USE_FAT_LINKAGES */
 }
 
 int sentence_disjunct_cost(Sentence sent, int i) {
@@ -930,7 +980,7 @@ int sentence_disjunct_cost(Sentence sent, int i) {
 
 	/* The sat solver (currently) fails to fill in link_info */
 	if (!sent->link_info) return 0;
-	return sent->link_info[i].disjunct_cost;
+	return (int) ceilf(sent->link_info[i].disjunct_cost);
 }
 
 int sentence_link_cost(Sentence sent, int i)
@@ -949,6 +999,101 @@ int sentence_nth_word_has_disjunction(Sentence sent, int i)
 	return (sent->parse_info->chosen_disjuncts[i] != NULL);
 }
 
+/* ============================================================== */
+/* A kind of morphism post-processing */
+
+#define INFIX_MARK '='
+
+/**
+ * This routine solves the problem of mis-link stem + suffix.
+ * It checks that the actual stem+suffix, when concatenated, restores
+ * the original word.  This is a work-around for somewhat sloppy
+ * dictionaries, which aren't entirely careful with making sure
+ * that the stem+suffix links are unique. Here's an illustrartion of
+ * the problem: The word Russian "тест" can be split into 
+ * тест.= =.ndmsi and also тес.= =т.amss The, during linkage, 
+ * тес.= and =.ndmsi are linked -- but this is not the original word; 
+ * its junk.  This routine marks such linkages as 'bad'.  Really, the
+ * dictionary should eb fixed ,, but what the hell. This isn't hard.
+ */
+static void sane_morphism(Sentence sent, Parse_Options opts)
+{
+	int lk, i;
+	Parse_info pi = sent->parse_info;
+
+	for (lk = 0; lk < sent->num_linkages_alloced; lk++)
+	{
+		Linkage_info *lifo = &sent->link_info[lk];
+		extract_links(lifo->index, sent->null_count, pi);
+		for (i=0; i<sent->length; i++)
+		{
+			const char *djw;
+			size_t len;
+			const char *unsplit = sent->word[i].unsplit_word;
+			size_t unlen;
+
+			/* Ignore island words */
+			if (NULL == pi->chosen_disjuncts[i])
+				continue;
+
+			/* Ignore suffixes, for now */
+			if (NULL == unsplit)
+				continue;
+
+			/* If its a perfect match, then keep going */
+			unlen = strlen(unsplit);
+			djw = pi->chosen_disjuncts[i]->string;
+			if (0 == strncmp(djw, unsplit, unlen))
+				continue;
+
+			/* Perhaps its a perfect match after capitalization. */
+			if (sent->word[i].firstupper)
+			{
+				char temp_word[MAX_WORD+1];
+				downcase_utf8_str(temp_word, unsplit, MAX_WORD);
+				if (0 == strncmp(djw, temp_word, strlen(temp_word)))
+					continue;
+			}
+
+			/* If the next word is a suffix, and, when united with
+			 * the stem, it recreates the original word, then we
+			 * are very happy, and keep going. */
+			len = strlen(djw);
+			if ((INFIX_MARK == djw[len-1]) &&
+			    ((i+1) < sent->length) && 
+			    (NULL != pi->chosen_disjuncts[i+1]) &&
+			    (INFIX_MARK == pi->chosen_disjuncts[i+1]->string[0]))
+			{
+				char *p;
+				char newword[MAX_WORD+1];
+				strcpy(newword, djw);
+				p = strrchr(newword, '.');
+				if (p) *p = 0x0;
+				strcat(newword, pi->chosen_disjuncts[i+1]->string+1);
+
+				/* OK, we built the concatenation .. does it match? */
+				if (0 == strncmp(newword, unsplit, unlen))
+					continue;
+
+				/* If we are here, it didn't match. Is that because of
+				 * capitalization? Lets check. */
+				if (sent->word[i].firstupper)
+				{
+					char temp_word[MAX_WORD+1];
+					downcase_utf8_str(temp_word, unsplit, MAX_WORD);
+					if (0 == strncmp(newword, temp_word, strlen(temp_word)))
+						continue;
+				}
+			}
+
+			/* Oh no ... we've joined together the stem and suffix incorrectly! */
+			sent->num_valid_linkages --;
+			lifo->N_violations ++;
+			break;
+		}
+	}
+}
+
 static void chart_parse(Sentence sent, Parse_Options opts)
 {
 	int nl;
@@ -994,6 +1139,7 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 		print_time(opts, "Counted parses");
 
 		post_process_linkages(sent, opts);
+		sane_morphism(sent, opts);
 		if (sent->num_valid_linkages > 0) break;
 
 		/* If we are here, then no valid linakges were found.
@@ -1092,14 +1238,16 @@ Linkage linkage_create(int k, Sentence sent, Parse_Options opts)
 
 	linkage->num_words = sent->length;
 	linkage->word = (const char **) exalloc(linkage->num_words*sizeof(char *));
-	linkage->current = 0;
-	linkage->num_sublinkages=0;
-	linkage->sublinkage = NULL;
-	linkage->unionized = FALSE;
 	linkage->sent = sent;
 	linkage->opts = opts;
 	linkage->info = &sent->link_info[k];
+#ifdef USE_FAT_LINKAGES
+	linkage->sublinkage = NULL;
+	linkage->unionized = FALSE;
+	linkage->current = 0;
+	linkage->num_sublinkages=0;
 	linkage->dis_con_tree = NULL;
+#endif /* USE_FAT_LINKAGES */
 
 	extract_links(sent->link_info[k].index, sent->null_count, sent->parse_info);
 	compute_chosen_words(sent, linkage);
@@ -1125,24 +1273,30 @@ Linkage linkage_create(int k, Sentence sent, Parse_Options opts)
 
 int linkage_get_current_sublinkage(const Linkage linkage)
 { 
+#ifdef USE_FAT_LINKAGES
 	return linkage->current; 
+#else
+	return 0;
+#endif /* USE_FAT_LINKAGES */
 } 
 
 int linkage_set_current_sublinkage(Linkage linkage, int index)
 {
+#ifdef USE_FAT_LINKAGES
 	if ((index < 0) ||
 		(index >= linkage->num_sublinkages))
 	{
 		return 0;
 	}
 	linkage->current = index;
+#endif /* USE_FAT_LINKAGES */
 	return 1;
 }
 
 static void exfree_pp_info(PP_info *ppi)
 {
 	if (ppi->num_domains > 0)
-		exfree(ppi->domain_name, sizeof(const char *)*ppi->num_domains);
+		exfree((void *) ppi->domain_name, sizeof(const char *) * ppi->num_domains);
 	ppi->domain_name = NULL;
 	ppi->num_domains = 0;
 }
@@ -1155,34 +1309,45 @@ void linkage_delete(Linkage linkage)
 	/* Can happen on panic timeout or user error */
 	if (NULL == linkage) return;
 
-	for (i=0; i<linkage->num_words; ++i)
+	for (i = 0; i < linkage->num_words; ++i)
 	{
 		exfree((void *) linkage->word[i], strlen(linkage->word[i])+1);
 	}
-	exfree(linkage->word, sizeof(char *)*linkage->num_words);
+	exfree((void *) linkage->word, sizeof(const char *) * linkage->num_words);
 
-	for (i=0; i<linkage->num_sublinkages; ++i)
+#ifdef USE_FAT_LINKAGES
+	for (i = 0; i < linkage->num_sublinkages; ++i)
+#endif /* USE_FAT_LINKAGES */
 	{
+#ifdef USE_FAT_LINKAGES
 		s = &(linkage->sublinkage[i]);
-		for (j=0; j<s->num_links; ++j) {
+#else
+		s = &(linkage->sublinkage);
+#endif /* USE_FAT_LINKAGES */
+		for (j = 0; j < s->num_links; ++j) {
 			exfree_link(s->link[j]);
 		}
-		exfree(s->link, sizeof(Link)*s->num_links);
+		exfree(s->link, sizeof(Link) * s->num_links);
 		if (s->pp_info != NULL) {
-			for (j=0; j<s->num_links; ++j) {
+			for (j = 0; j < s->num_links; ++j) {
 				exfree_pp_info(&s->pp_info[j]);
 			}
-			exfree(s->pp_info, sizeof(PP_info)*s->num_links);
+			exfree(s->pp_info, sizeof(PP_info) * s->num_links);
 			s->pp_info = NULL;
 			post_process_free_data(&s->pp_data);
 		}
 		if (s->violation != NULL) {
-			exfree((void *) s->violation, sizeof(char)*(strlen(s->violation)+1));
+			exfree((void *) s->violation, sizeof(char) * (strlen(s->violation)+1));
 		}
 	}
+#ifdef USE_FAT_LINKAGES
 	exfree(linkage->sublinkage, sizeof(Sublinkage)*linkage->num_sublinkages);
+#endif /* USE_FAT_LINKAGES */
+
+#ifdef USE_FAT_LINKAGES
 	if (linkage->dis_con_tree)
 		free_DIS_tree(linkage->dis_con_tree);
+#endif /* USE_FAT_LINKAGES */
 	exfree(linkage, sizeof(struct Linkage_s));
 }
 
@@ -1306,7 +1471,11 @@ int linkage_compute_union(Linkage linkage)
 
 int linkage_get_num_sublinkages(const Linkage linkage)
 {
+#ifdef USE_FAT_LINKAGES
 	return linkage->num_sublinkages;
+#else
+	return 1;
+#endif /* USE_FAT_LINKAGES */
 }
 
 int linkage_get_num_words(const Linkage linkage)
@@ -1316,14 +1485,22 @@ int linkage_get_num_words(const Linkage linkage)
 
 int linkage_get_num_links(const Linkage linkage)
 {
+#ifdef USE_FAT_LINKAGES
 	int current = linkage->current;
 	return linkage->sublinkage[current].num_links;
+#else
+	return linkage->sublinkage.num_links;
+#endif /* USE_FAT_LINKAGES */
 }
 
 static inline int verify_link_index(const Linkage linkage, int index)
 {
 	if ((index < 0) ||
+#ifdef USE_FAT_LINKAGES
 		(index >= linkage->sublinkage[linkage->current].num_links))
+#else
+		(index >= linkage->sublinkage.num_links))
+#endif /* USE_FAT_LINKAGES */
 	{
 		return 0;
 	}
@@ -1335,7 +1512,9 @@ int linkage_get_link_length(const Linkage linkage, int index)
 	Link *link;
 	int word_has_link[MAX_SENTENCE];
 	int i, length;
+#ifdef USE_FAT_LINKAGES
 	int current = linkage->current;
+#endif /* USE_FAT_LINKAGES */
 
 	if (!verify_link_index(linkage, index)) return -1;
 
@@ -1343,13 +1522,24 @@ int linkage_get_link_length(const Linkage linkage, int index)
 		word_has_link[i] = FALSE;
 	}
 
-	for (i=0; i<linkage->sublinkage[current].num_links; ++i) {
+#ifdef USE_FAT_LINKAGES
+	for (i=0; i<linkage->sublinkage[current].num_links; ++i)
+	{
 		link = linkage->sublinkage[current].link[i];
 		word_has_link[link->l] = TRUE;
 		word_has_link[link->r] = TRUE;
 	}
-
 	link = linkage->sublinkage[current].link[index];
+#else
+	for (i=0; i<linkage->sublinkage.num_links; ++i)
+	{
+		link = linkage->sublinkage.link[i];
+		word_has_link[link->l] = TRUE;
+		word_has_link[link->r] = TRUE;
+	}
+	link = linkage->sublinkage.link[index];
+#endif /* USE_FAT_LINKAGES */
+
 	length = link->r - link->l;
 	for (i= link->l+1; i < link->r; ++i) {
 		if (!word_has_link[i]) length--;
@@ -1361,7 +1551,11 @@ int linkage_get_link_lword(const Linkage linkage, int index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return -1;
+#ifdef USE_FAT_LINKAGES
 	link = linkage->sublinkage[linkage->current].link[index];
+#else
+	link = linkage->sublinkage.link[index];
+#endif /* USE_FAT_LINKAGES */
 	return link->l;
 }
 
@@ -1369,7 +1563,11 @@ int linkage_get_link_rword(const Linkage linkage, int index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return -1;
+#ifdef USE_FAT_LINKAGES
 	link = linkage->sublinkage[linkage->current].link[index];
+#else
+	link = linkage->sublinkage.link[index];
+#endif /* USE_FAT_LINKAGES */
 	return link->r;
 }
 
@@ -1377,7 +1575,11 @@ const char * linkage_get_link_label(const Linkage linkage, int index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return NULL;
+#ifdef USE_FAT_LINKAGES
 	link = linkage->sublinkage[linkage->current].link[index];
+#else
+	link = linkage->sublinkage.link[index];
+#endif /* USE_FAT_LINKAGES */
 	return link->name;
 }
 
@@ -1385,7 +1587,11 @@ const char * linkage_get_link_llabel(const Linkage linkage, int index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return NULL;
+#ifdef USE_FAT_LINKAGES
 	link = linkage->sublinkage[linkage->current].link[index];
+#else
+	link = linkage->sublinkage.link[index];
+#endif /* USE_FAT_LINKAGES */
 	return link->lc->string;
 }
 
@@ -1393,7 +1599,11 @@ const char * linkage_get_link_rlabel(const Linkage linkage, int index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return NULL;
+#ifdef USE_FAT_LINKAGES
 	link = linkage->sublinkage[linkage->current].link[index];
+#else
+	link = linkage->sublinkage.link[index];
+#endif /* USE_FAT_LINKAGES */
 	return link->rc->string;
 }
 
@@ -1472,7 +1682,7 @@ int linkage_is_fat(const Linkage linkage)
 #endif /* USE_FAT_LINKAGES */
 }
 
-int linkage_and_cost(Linkage linkage)
+int linkage_and_cost(const Linkage linkage)
 {
 #ifdef USE_FAT_LINKAGES
 	/* The sat solver (currently) fails to fill in info */
@@ -1501,7 +1711,11 @@ int linkage_get_link_num_domains(const Linkage linkage, int index)
 {
 	PP_info *pp_info;
 	if (!verify_link_index(linkage, index)) return -1;
+#ifdef USE_FAT_LINKAGES
 	pp_info = &linkage->sublinkage[linkage->current].pp_info[index];
+#else
+	pp_info = &linkage->sublinkage.pp_info[index];
+#endif /* USE_FAT_LINKAGES */
 	return pp_info->num_domains;
 }
 
@@ -1509,20 +1723,32 @@ const char ** linkage_get_link_domain_names(const Linkage linkage, int index)
 {
 	PP_info *pp_info;
 	if (!verify_link_index(linkage, index)) return NULL;
+#ifdef USE_FAT_LINKAGES
 	pp_info = &linkage->sublinkage[linkage->current].pp_info[index];
+#else
+	pp_info = &linkage->sublinkage.pp_info[index];
+#endif /* USE_FAT_LINKAGES */
 	return pp_info->domain_name;
 }
 
 const char * linkage_get_violation_name(const Linkage linkage)
 {
+#ifdef USE_FAT_LINKAGES
 	return linkage->sublinkage[linkage->current].violation;
+#else
+	return linkage->sublinkage.violation;
+#endif /* USE_FAT_LINKAGES */
 }
 
 int linkage_is_canonical(const Linkage linkage)
 {
+#ifdef USE_FAT_LINKAGES
 	/* The sat solver (currently) fails to fill in info */
 	if (!linkage->info) return TRUE;
 	return linkage->info->canonical;
+#else
+	return TRUE;
+#endif /* USE_FAT_LINKAGES */
 }
 
 int linkage_is_improper(const Linkage linkage)
@@ -1538,24 +1764,37 @@ int linkage_is_improper(const Linkage linkage)
 
 int linkage_has_inconsistent_domains(const Linkage linkage)
 {
+#ifdef USE_FAT_LINKAGES
 	/* The sat solver (currently) fails to fill in info */
 	if (!linkage->info) return FALSE;
 	return linkage->info->inconsistent_domains;
+#else
+	return FALSE;
+#endif /* USE_FAT_LINKAGES */
 }
 
 void linkage_post_process(Linkage linkage, Postprocessor * postprocessor)
 {
+#ifdef USE_FAT_LINKAGES
 	int N_sublinkages = linkage_get_num_sublinkages(linkage);
+#endif /* USE_FAT_LINKAGES */
 	Parse_Options opts = linkage->opts;
 	Sentence sent = linkage->sent;
 	Sublinkage * subl;
 	PP_node * pp;
-	int i, j, k;
+	int j, k;
 	D_type_list * d;
 
+#ifdef USE_FAT_LINKAGES
+	int i;
 	for (i = 0; i < N_sublinkages; ++i)
+#endif /* USE_FAT_LINKAGES */
 	{
+#ifdef USE_FAT_LINKAGES
 		subl = &linkage->sublinkage[i];
+#else
+		subl = &linkage->sublinkage;
+#endif /* USE_FAT_LINKAGES */
 		if (subl->pp_info != NULL)
 		{
 			for (j = 0; j < subl->num_links; ++j)
@@ -1563,9 +1802,9 @@ void linkage_post_process(Linkage linkage, Postprocessor * postprocessor)
 				exfree_pp_info(&subl->pp_info[j]);
 			}
 			post_process_free_data(&subl->pp_data);
-			exfree(subl->pp_info, sizeof(PP_info)*subl->num_links);
+			exfree(subl->pp_info, sizeof(PP_info) * subl->num_links);
 		}
-		subl->pp_info = (PP_info *) exalloc(sizeof(PP_info)*subl->num_links);
+		subl->pp_info = (PP_info *) exalloc(sizeof(PP_info) * subl->num_links);
 		for (j = 0; j < subl->num_links; ++j)
 		{
 			subl->pp_info[j].num_domains = 0;
@@ -1573,7 +1812,7 @@ void linkage_post_process(Linkage linkage, Postprocessor * postprocessor)
 		}
 		if (subl->violation != NULL)
 		{
-			exfree((void *)subl->violation, sizeof(char)*(strlen(subl->violation)+1));
+			exfree((void *)subl->violation, sizeof(char) * (strlen(subl->violation)+1));
 			subl->violation = NULL;
 		}
 
