@@ -89,11 +89,6 @@ static void init_table(Sentence sent)
 	memset(ctxt->table, 0, ctxt->table_size*sizeof(Table_connector*));
 }
 
-int x_match(Sentence sent, Connector *a, Connector *b)
-{
-	return do_match(sent, a, b, 0, 0);
-}
-
 #ifdef USE_FAT_LINKAGES
 void count_set_effective_distance(Sentence sent)
 {
@@ -104,7 +99,6 @@ void count_unset_effective_distance(Sentence sent)
 {
 	sent->count_ctxt->effective_dist = NULL;
 }
-#endif /* USE_FAT_LINKAGES */
 
 /*
  * Returns TRUE if s and t match according to the connector matching
@@ -123,9 +117,7 @@ int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
 {
 	const char *s, *t;
 	int x, y;
-#ifdef USE_FAT_LINKAGES
 	int dist;
-#endif /* USE_FAT_LINKAGES */
 
 	if (a->label != b->label) return FALSE;
 
@@ -142,8 +134,6 @@ int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
 	x = a->priority;
 	y = b->priority;
 
-#ifdef USE_FAT_LINKAGES
-
 	/* Probably not necessary, as long as 
 	 * effective_dist[0][0]=0 and is defined */
 	if (aw == 0 && bw == 0) {
@@ -157,7 +147,6 @@ int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
 	/*	printf("M: a=%4s b=%4s  ap=%d bp=%d  aw=%d  bw=%d  a->ll=%d b->ll=%d  dist=%d\n",
 		   s, t, x, y, aw, bw, a->length_limit, b->length_limit, dist); */
 	if (dist > a->length_limit || dist > b->length_limit) return FALSE;
-#endif /* USE_FAT_LINKAGES */
 
 	if ((x == THIN_priority) && (y == THIN_priority))
 	{
@@ -169,12 +158,8 @@ int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
 		   */
 		while ((*s != '\0') && (*t != '\0'))
 		{
-#ifdef USE_FAT_LINKAGES
 			if ((*s == '*') || (*t == '*') ||
-				((*s == *t) && (*s != '^')))
-#else
-			if ((*s == '*') || (*t == '*') || (*s == *t))
-#endif /* USE_FAT_LINKAGES */
+			   ((*s == *t) && (*s != '^')))
 			{
 				s++;
 				t++;
@@ -199,11 +184,7 @@ int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
 		   */
 		while ((*s != '\0') && (*t != '\0'))
 		{
-#ifdef USE_FAT_LINKAGES
 			if ((*s == *t) || (*s == '*') || (*t == '^'))
-#else
-			if ((*s == *t) || (*s == '*'))
-#endif /* USE_FAT_LINKAGES */
 			{
 				s++;
 				t++;
@@ -215,11 +196,7 @@ int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
 	{
 		while ((*s != '\0') && (*t != '\0'))
 		{
-#ifdef USE_FAT_LINKAGES
 			if ((*s == *t) || (*t == '*') || (*s == '^'))
-#else
-			if ((*s == *t) || (*t == '*'))
-#endif /* USE_FAT_LINKAGES */
 			{
 				s++;
 				t++;
@@ -233,6 +210,24 @@ int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
 		return FALSE;
 }
 
+#else /* not USE_FAT_LINKAGES */
+
+/*
+ * Returns TRUE if s and t match according to the connector matching
+ * rules.  The connector strings must be properly formed, starting with
+ * zero or more upper case letters, followed by some other letters, and
+ * The algorithm is symmetric with respect to a and b.
+ *
+ * It works as follows:  The labels must match.  
+ * The sequence of upper case letters must match exactly.  After these comes
+ * a sequence of lower case letters or "*"s. 
+ */
+int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
+{
+   return prune_match(bw - aw, a, b);
+}
+#endif /* not USE_FAT_LINKAGES */
+
 /** 
  * Stores the value in the table.  Assumes it's not already there.
  */
@@ -320,9 +315,9 @@ static s64 pseudocount(Sentence sent,
 static s64 do_count(Sentence sent, int lw, int rw,
                     Connector *le, Connector *re, int null_count)
 {
-	s64 total, pseudototal;
+	s64 total;
 	int start_word, end_word, w;
-	s64 leftcount, rightcount;
+	s64 leftcount, rightcount, pseudototal;
 	int lcost, rcost, Lmatch, Rmatch;
 
 	Match_node * m, *m1;
