@@ -64,10 +64,11 @@ static const char * rpunc_con = "RPUNC";
 static const char * lpunc_con = "LPUNC";
 static const char * units_con = "UNITS";
 
-/* Hmmmm SUF and PRE do not seem to be used at this time ... */
+/* SUF is used in the Russian dict; PRE is not used anywhere, yet ... */
 static const char * suf_con = "SUF";
 static const char * pre_con = "PRE";
 
+
 static void count_affix(Dictionary dict, Dict_node *dn)
 {
 	if (word_has_connector(dn, rpunc_con, '+')) dict->r_strippable++;
@@ -102,11 +103,6 @@ static void load_affix(Dictionary dict, Dict_node *dn)
 	}
 	if (word_has_connector(dn, suf_con, '+'))
 	{
-		if (0 == strcmp("=", dn->string))
-		{
-			dn->string = string_set_add("", dict->string_set);
-			dict->have_empty_suffix = TRUE;
-		}
 		dict->suffix[k] = dn->string;
 		k++;
 	}
@@ -169,6 +165,10 @@ static void affix_list_delete(Dictionary dict)
 	xfree(dict->prefix, dict->p_strippable * sizeof(char *));
 }
 
+static Dictionary
+dictionary_six(const char * lang, const char * dict_name,
+                const char * pp_name, const char * cons_name,
+                const char * affix_name, const char * regex_name);
 /**
  * Read dictionary entries from a wide-character string "input".
  * All other parts are read from files.
@@ -221,7 +221,6 @@ dictionary_six_str(const char * lang,
 	dict->input = NULL;
 
 	dict->affix_table = NULL;
-	dict->have_empty_suffix = FALSE;
 	if (affix_name != NULL)
 	{
 		dict->affix_table = dictionary_six(lang, affix_name, NULL, NULL, NULL, NULL);
@@ -252,18 +251,23 @@ dictionary_six_str(const char * lang,
 
 	dict->left_wall_defined  = boolean_dictionary_lookup(dict, LEFT_WALL_WORD);
 	dict->right_wall_defined = boolean_dictionary_lookup(dict, RIGHT_WALL_WORD);
+
+	dict->empty_word_defined = boolean_dictionary_lookup(dict, EMPTY_WORD);
+
 	dict->postprocessor	  = post_process_open(pp_name);
 	dict->constituent_pp	 = post_process_open(cons_name);
 
 	dict->unknown_word_defined = boolean_dictionary_lookup(dict, UNKNOWN_WORD);
 	dict->use_unknown_word = TRUE;
 
+#ifdef USE_FAT_LINKAGES
 	if ((dict_node = dictionary_lookup_list(dict, ANDABLE_CONNECTORS_WORD)) != NULL) {
 		dict->andable_connector_set = connector_set_create(dict_node->exp);
 	} else {
 		dict->andable_connector_set = NULL;
 	}
 	free_lookup_list(dict_node);
+#endif /* USE_FAT_LINKAGES */
 
 	if ((dict_node = dictionary_lookup_list(dict, UNLIMITED_CONNECTORS_WORD)) != NULL) {
 		dict->unlimited_connector_set = connector_set_create(dict_node->exp);
@@ -283,7 +287,7 @@ failure:
 /**
  * Use filenames of six different files to put together the dictionary.
  */
-Dictionary
+static Dictionary
 dictionary_six(const char * lang, const char * dict_name,
                 const char * pp_name, const char * cons_name,
                 const char * affix_name, const char * regex_name)
@@ -430,7 +434,9 @@ int dictionary_delete(Dictionary dict)
 	}
 	spellcheck_destroy(dict->spell_checker);
 
+#ifdef USE_FAT_LINKAGES
 	connector_set_delete(dict->andable_connector_set);
+#endif /* USE_FAT_LINKAGES */
 	connector_set_delete(dict->unlimited_connector_set);
 
 	post_process_close(dict->postprocessor);
