@@ -20,6 +20,9 @@
 #include "jni-client.h"
 #include "utilities.h"
 
+/* Default to the english language. */
+static const char* in_language = "en";
+
 typedef struct
 {
 	Dictionary    dict;
@@ -50,6 +53,7 @@ static void java_key_alloc(void)
 static per_thread_data * global_ptd = NULL;
 #endif
 
+
 static per_thread_data * get_ptd(JNIEnv *env, jclass cls)
 {
 #ifdef USE_PTHREADS
@@ -72,7 +76,9 @@ static void setup_panic_parse_options(Parse_Options opts)
 	parse_options_set_min_null_count(opts, 1);
 	parse_options_set_max_null_count(opts, MAX_SENTENCE);
 	parse_options_set_max_parse_time(opts, 60);
+#ifdef USE_FAT_LINKAGES
 	parse_options_set_use_fat_links(opts, FALSE);
+#endif /* USE_FAT_LINKAGES */
 	parse_options_set_islands_ok(opts, TRUE);
 	parse_options_set_short_length(opts, 6);
 	parse_options_set_all_short_connectors(opts, TRUE);
@@ -142,10 +148,7 @@ static per_thread_data * init(JNIEnv *env, jclass cls)
 	parse_options_set_verbosity(ptd->opts,0);
 	parse_options_set_spell_guess(ptd->opts, FALSE);
 
-	/* Default to the english language; will need to fix
-	 * this if/when more languages are supported.
-	 */
-	ptd->dict = dictionary_create_lang("en");
+	ptd->dict = dictionary_create_lang(in_language);
 	if (!ptd->dict) throwException(env, "Error: unable to open dictionary");
 	else test();
 
@@ -258,7 +261,9 @@ static void jParse(JNIEnv *env, per_thread_data *ptd, char* inputString)
 	parse_options_set_disjunct_costf(opts, 2.0f);
 	parse_options_set_min_null_count(opts, 0);
 	parse_options_set_max_null_count(opts, 0);
+#ifdef USE_FAT_LINKAGES
 	parse_options_set_use_fat_links(opts, FALSE);
+#endif /* USE_FAT_LINKAGES */
 	parse_options_reset_resources(opts);
 
 	ptd->num_linkages = sentence_parse(ptd->sent, ptd->opts);
@@ -306,9 +311,11 @@ static void makeLinkage(per_thread_data *ptd)
 			linkage_delete(ptd->linkage);
 
 		ptd->linkage = linkage_create(ptd->cur_linkage,ptd->sent,ptd->opts);
+#ifdef USE_FAT_LINKAGES
 		linkage_compute_union(ptd->linkage);
 		linkage_set_current_sublinkage(ptd->linkage,
 		                        linkage_get_num_sublinkages(ptd->linkage)-1);
+#endif /* USE_FAT_LINKAGES */
 
 #if DO_PHRASE_TREE
 		if (tree)
@@ -372,6 +379,29 @@ Java_org_linkgrammar_LinkGrammar_setDictionariesPath(JNIEnv *env,
 	(*env)->ReleaseStringUTFChars(env,path, nativePath);
 }
 
+/*
+ * Class:      LinkGrammar
+ * Method:     setLanguage
+ * Signature: (Ljava/lang/String;)V
+ */
+JNIEXPORT void JNICALL
+Java_org_linkgrammar_LinkGrammar_setLanguage(JNIEnv *env, jclass cls, jstring str)
+{
+	in_language = (*env)->GetStringUTFChars(env,str,0);
+}
+
+/*
+ * Class:      LinkGrammar
+ * Method:     setMaxLinkages
+ * Signature: (I)V
+ */
+JNIEXPORT void JNICALL
+Java_org_linkgrammar_LinkGrammar_setMaxLinkages(JNIEnv *env, jclass cls, jint maxLinkages)
+{
+	per_thread_data *ptd = get_ptd(env, cls);
+	parse_options_set_linkage_limit(ptd->opts, maxLinkages);
+}
+
 /*
  * Class:      LinkGrammar
  * Method:     init
@@ -436,6 +466,8 @@ Java_org_linkgrammar_LinkGrammar_getNumWords(JNIEnv *env, jclass cls)
 }
 
 /*
+ * This is making a call to a deprecated API function. The coresponding
+ * java function should be marked deprecated as well.
  * Class:      LinkGrammar
  * Method:     getWord
  * Signature: (I)Ljava/lang/String;
