@@ -1,6 +1,7 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* Copyright 2013 Linas Vepstas                                          */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
@@ -13,16 +14,11 @@
 #ifndef _LINKINCLUDESH_
 #define _LINKINCLUDESH_
 
+#include <stdio.h>  /* Needed for FILE* below */
 #include <link-grammar/link-features.h>
 
 LINK_BEGIN_DECLS
 
-#if  __GNUC__ > 2
-#define GNUC_DEPRECATED __attribute__((deprecated))
-#else
-#define GNUC_DEPRECATED
-#endif
-
 /**********************************************************************
  *
  * System initialization
@@ -132,10 +128,6 @@ link_public_api(void)
      parse_options_set_cost_model_type(Parse_Options opts, Cost_Model_type cm);
 link_public_api(Cost_Model_type)
      parse_options_get_cost_model_type(Parse_Options opts);
-link_public_api(void)
-     parse_options_set_use_fat_links(Parse_Options opts, int use_fat_links);
-link_public_api(int)
-     parse_options_get_use_fat_links(Parse_Options opts);
 link_public_api(void)
      parse_options_set_use_sat_parser(Parse_Options opts, int use_sat_solver);
 link_public_api(int)
@@ -231,9 +223,9 @@ link_public_api(void)
 link_public_api(int)
      parse_options_get_display_senses(Parse_Options opts);
 link_public_api(void)
-     parse_options_set_display_union(Parse_Options opts, int val);
+     parse_options_set_display_suffixes(Parse_Options opts, int val);
 link_public_api(int)
-     parse_options_get_display_union(Parse_Options opts);
+     parse_options_get_display_suffixes(Parse_Options opts);
 link_public_api(void)
      parse_options_set_echo_on(Parse_Options opts, int val);
 link_public_api(int)
@@ -265,20 +257,13 @@ link_public_api(int)
      sentence_num_valid_linkages(Sentence sent);
 link_public_api(int)
      sentence_num_linkages_post_processed(Sentence sent);
-link_public_api(int)
-     sentence_num_thin_linkages(Sentence sent);
 link_public_api(int)
      sentence_num_violations(Sentence sent, int i);
-link_public_api(int)
-     sentence_and_cost(Sentence sent, int i);
 link_public_api(int)
      sentence_disjunct_cost(Sentence sent, int i);
 link_public_api(int)
      sentence_link_cost(Sentence sent, int i);
 
-link_public_api(int)
-     sentence_contains_conjunction(Sentence sent);
-
 /**********************************************************************
  *
  * Functions that create and manipulate Linkages.
@@ -293,16 +278,10 @@ typedef struct Linkage_s * Linkage;
 
 link_public_api(Linkage)
      linkage_create(int index, Sentence sent, Parse_Options opts);
-link_public_api(int) 
-     linkage_get_current_sublinkage(const Linkage linkage); 
-link_public_api(int)
-     linkage_set_current_sublinkage(Linkage linkage, int index);
 link_public_api(void)
      linkage_delete(Linkage linkage);
 link_public_api(Sentence)
      linkage_get_sentence(const Linkage linkage);
-link_public_api(int)
-     linkage_get_num_sublinkages(const Linkage linkage);
 link_public_api(int)
      linkage_get_num_words(const Linkage linkage);
 link_public_api(int)
@@ -357,26 +336,14 @@ link_public_api(char *)
      linkage_print_diagram(const Linkage linkage);
 link_public_api(void)
      linkage_free_diagram(char * str);
-link_public_api(int)
-     linkage_compute_union(Linkage linkage);
-link_public_api(int)
-     linkage_is_fat(const Linkage linkage);
 link_public_api(int)
      linkage_unused_word_cost(const Linkage linkage);
 link_public_api(int)
      linkage_disjunct_cost(const Linkage linkage);
-link_public_api(int)
-     linkage_and_cost(const Linkage linkage);
 link_public_api(int)
      linkage_link_cost(const Linkage linkage);
 link_public_api(double)
      linkage_corpus_cost(const Linkage linkage);
-link_public_api(int)
-     linkage_is_canonical(const Linkage linkage);
-link_public_api(int)
-     linkage_is_improper(const Linkage linkage);
-link_public_api(int)
-     linkage_has_inconsistent_domains(const Linkage linkage);
 link_public_api(const char *)
      linkage_get_violation_name(const Linkage linkage);
 
@@ -422,7 +389,25 @@ link_public_api(int)
 /**********************************************************************
  *
  * Internal functions -- do not use these in new code!
- * These are not intended for general public use, but are required to 
+ * These are not intended for general public use, but are required to
+ * get the link-parser executable to link under MSVC6.
+ *
+ ***********************************************************************/
+
+link_public_api(void)
+     dict_display_word_expr(Dictionary dict, const char *);
+link_public_api(void)
+     dict_display_word_info(Dictionary dict, const char *);
+link_public_api(void)
+     left_print_string(FILE* fp, const char *, const char *);
+link_public_api(int)
+     lg_expand_disjunct_list(Sentence sent);
+
+
+/**********************************************************************
+ *
+ * Internal functions -- do not use these in new code!
+ * These are not intended for general public use, but are required to
  * work around certain Micorsoft Windows linking oddities
  * (specifically, to be callable from the JNI bindings library.)
  *
@@ -448,35 +433,109 @@ link_public_api(void)
  *
  ********************************************************/
 
+#if  __GNUC__ > 2
+#define GNUC_DEPRECATED __attribute__((deprecated))
+#else
+#define GNUC_DEPRECATED
+#endif
+
+#ifdef _MSC_VER
+#define MS_DEPRECATED __declspec(deprecated)
+#else
+#define MS_DEPRECATED
+#endif
+
+#ifdef USE_FAT_LINKAGES
+#pragma message("WARNING: Support for FAT linakges is going away!")
+#endif /* USE_FAT_LINKAGES */
+
+/* When fat links are gone, there's no union to compute. */
+MS_DEPRECATED link_public_api(int)
+     linkage_compute_union(Linkage linkage) GNUC_DEPRECATED;
+
+/* When fat links go away, all linkages are thin */
+MS_DEPRECATED link_public_api(int)
+     linkage_is_fat(const Linkage linkage) GNUC_DEPRECATED;
+
+/* When fat links go away, the number of sublinkages will always be one */
+MS_DEPRECATED link_public_api(int)
+     linkage_get_num_sublinkages(const Linkage linkage) GNUC_DEPRECATED;
+
+/* When fat links go away, the only valid sublinkage will be zero. */
+MS_DEPRECATED link_public_api(int)
+     linkage_set_current_sublinkage(Linkage linkage, int index) GNUC_DEPRECATED;
+
+/* When fat links go away, the only valid sublinkage will be zero. */
+MS_DEPRECATED link_public_api(int)
+     linkage_get_current_sublinkage(const Linkage linkage) GNUC_DEPRECATED;
+
+/* When fat links go away, all linkages are proper. */
+MS_DEPRECATED link_public_api(int)
+     linkage_is_improper(const Linkage linkage) GNUC_DEPRECATED;
+
+/* All linkages are canonical when there are no fat links. */
+MS_DEPRECATED link_public_api(int)
+     linkage_is_canonical(const Linkage linkage) GNUC_DEPRECATED;
+
+/* Not possible without fat links */
+MS_DEPRECATED link_public_api(int)
+     linkage_has_inconsistent_domains(const Linkage linkage) GNUC_DEPRECATED;
+
+/* and cost is always zero without fat links */
+MS_DEPRECATED link_public_api(int)
+     linkage_and_cost(const Linkage linkage) GNUC_DEPRECATED;
+
+/* All linkages are thin, when none are fat */
+MS_DEPRECATED link_public_api(int)
+     sentence_num_thin_linkages(Sentence sent) GNUC_DEPRECATED;
+
+/* Bogus/pointless when no fat links */
+MS_DEPRECATED link_public_api(int)
+     sentence_contains_conjunction(Sentence sent) GNUC_DEPRECATED;
+
+/* and cost is always zero without fat links */
+MS_DEPRECATED link_public_api(int)
+     sentence_and_cost(Sentence sent, int i) GNUC_DEPRECATED;
+
+/* Fat linkages will be going away "real soon now" */
+MS_DEPRECATED link_public_api(void)
+     parse_options_set_use_fat_links(Parse_Options opts, int use_fat_links) GNUC_DEPRECATED;
+MS_DEPRECATED link_public_api(int)
+     parse_options_get_use_fat_links(Parse_Options opts) GNUC_DEPRECATED;
+MS_DEPRECATED link_public_api(void)
+     parse_options_set_display_union(Parse_Options opts, int val) GNUC_DEPRECATED;
+MS_DEPRECATED link_public_api(int)
+     parse_options_get_display_union(Parse_Options opts) GNUC_DEPRECATED;
+
 /* Fails to include the regex file name, which is needed in any
  * practical application.  Thus, this call is deprecated.
  * XXX TBD: this will go away in Version 5.0. */
-link_public_api(Dictionary)
+MS_DEPRECATED link_public_api(Dictionary)
      dictionary_create(const char * dict_name,
                        const char * pp_name,
                        const char * cons_name,
                        const char * affix_name) GNUC_DEPRECATED;
 
 /* Both are deprecated, exported only for backwards-compat w/Java API. */
-link_public_api(int)
+MS_DEPRECATED link_public_api(int)
      dictionary_is_past_tense_form(Dictionary dict, const char * str) GNUC_DEPRECATED;
-link_public_api(int)
+MS_DEPRECATED link_public_api(int)
      dictionary_is_entity(Dictionary dict, const char * str) GNUC_DEPRECATED;
 
 /* For languages with regular prefix/suffix structure, this
  * function returrns misleading information. Do not use it
  * in new code! use linkage_get_word() instead! */
-link_public_api(const char *)
+MS_DEPRECATED link_public_api(const char *)
      sentence_get_word(Sentence sent, int wordnum) GNUC_DEPRECATED;
 
 /* Identical to sentence_get_word()
  * XXX TBD: make this go away in Version 5.0. */
-link_public_api(const char *)
+MS_DEPRECATED link_public_api(const char *)
      sentence_get_nth_word(Sentence sent, int i) GNUC_DEPRECATED;
 
 /* Who uses this function, anyway? How did this get exported?
  * XXX TBD: make this go away in Version 5.0. */
-link_public_api(int)
+MS_DEPRECATED link_public_api(int)
      sentence_nth_word_has_disjunction(Sentence sent, int i) GNUC_DEPRECATED;
 
 /* This is not intended for general use; its specific to the internals
@@ -487,12 +546,11 @@ link_public_api(int)
 
 /* These are obsolete, and do nothing.
  * XXX TBD: make these go away in Version 5.0. */
-link_public_api(void)
+MS_DEPRECATED link_public_api(void)
      lperror_clear(void) GNUC_DEPRECATED;
-extern link_public_api(int)
-     lperrno;
-extern link_public_api(char)
-     lperrmsg[];
+
+extern link_public_api(int) lperrno;
+extern link_public_api(char) lperrmsg[];
 
 LINK_END_DECLS
 
