@@ -1,6 +1,7 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* Copyright 2013 Linas Vepstas                                          */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
@@ -17,17 +18,43 @@
 #include "corpus/corpus.h"
 #include "print-util.h"
 
-const char * trailer(int mode);
-const char * header(int mode);
+#define LEFT_WALL_DISPLAY  ("LEFT-WALL")   /* the string to use to show the wall */
+#define LEFT_WALL_SUPPRESS ("Wd") /* If this connector is used on the wall, */
+                                  /* then suppress the display of the wall. */
+#define RIGHT_WALL_DISPLAY  ("RIGHT-WALL")   /* the string to use to show the wall */
+#define RIGHT_WALL_SUPPRESS ("RW") /* If this connector is used on the wall, */
 
-static void set_centers(const Linkage linkage, int center[], int print_word_0, int N_words_to_print)
+/* The Russian dictionary makes use of the empty word to deal with
+ * the splitting of words into variable-length word-counts */
+#define EMPTY_WORD ("=.zzz") /* pure whitespace */
+#define EMPTY_WORD_SUPPRESS ("ZZZ") /* link to pure whitespace */
+
+#define SUFFIX_WORD ("=")  /* suffixes start with this */
+#define SUFFIX_WORD_L 1     /* length of above */
+#define SUFFIX_SUPPRESS ("LL") /* suffix links start with this */
+#define SUFFIX_SUPPRESS_L 2  /* length of above */
+
+#define HIDE_SUFFIX   (!display_suffixes)
+
+
+static void set_centers(const Linkage linkage, int center[], Boolean print_word_0, int N_words_to_print)
 {
 	int i, len, tot;
+	Boolean display_suffixes = linkage->opts->display_suffixes;
 
 	tot = 0;
 	if (print_word_0) i=0; else i=1;
 	for (; i < N_words_to_print; i++)
 	{
+		/* Ignore suffixes */
+		if (HIDE_SUFFIX &&
+		    0 == strncmp (SUFFIX_WORD, linkage->word[i], SUFFIX_WORD_L))
+		{
+			center[i] = tot;
+			tot++;  // hack alert -- a trailing blank gets printed anyway ...
+			continue;
+		}
+
 		/* Centers obtained by counting the characters, 
 		 * not the bytes in the string.
 		 * len = strlen(linkage->word[i]);
@@ -222,13 +249,19 @@ char * linkage_print_disjuncts(const Linkage linkage)
 }
 
 /**
+ * XXX TODO: port over the suppression of ZZZ links, and also
+ * the HIDE_SUFFIX stuff, fom the ascii printing code, way below.
  */
 static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *pctx)
 {
 	int link, i,j;
 	int d;
 	int print_word_0 = 0, print_word_N = 0, N_wall_connectors, suppressor_used;
+#ifdef USE_FAT_LINKAGES
 	Sublinkage *sublinkage = &(linkage->sublinkage[linkage->current]);
+#else
+	Sublinkage *sublinkage = &(linkage->sublinkage);
+#endif /* USE_FAT_LINKAGES */
 	int N_links = sublinkage->num_links;
 	Link **ppla = sublinkage->link;
 	String  * string;
@@ -302,11 +335,16 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 		append_string(string,"[%d %d %d",
 				ppla[link]->l-d, ppla[link]->r-d, 
 				pctx->link_heights[link]);
+#ifdef USE_FAT_LINKAGES
 		if (ppla[link]->lc->label < 0) {
 			append_string(string," (%s)]", ppla[link]->name);
 		} else {
 			append_string(string," ()]");
 		}
+#else
+		append_string(string," (%s)]", ppla[link]->name);
+#endif /* USE_FAT_LINKAGES */
+
 	}
 	append_string(string,"]");
 	append_string(string,"\n");
@@ -334,43 +372,88 @@ void compute_chosen_words(Sentence sent, Linkage linkage)
 	Parse_info pi = sent->parse_info;
 	const char * chosen_words[MAX_SENTENCE];
 	Parse_Options opts = linkage->opts;
+	Boolean display_suffixes = opts->display_suffixes;
 
 	for (i=0; i<sent->length; i++)
 	{
 		const char *t;
+		/* If chosen_disjuncts is NULL, then this is an 'island' word
+		 * that has not been linked to. */
 		if (pi->chosen_disjuncts[i] == NULL)
 		{
-			/* no disjunct is used on this word because of null-links */
-
-			/* Well, alternative[0] is not really right, but what else can we print? */
-			t = sent->word[i].alternatives[0];
-			l = strlen(t) + 2;
-			s = (char *) xalloc(l+1);
-			sprintf(s, "[%s]", t);
-			t = string_set_add(s, sent->string_set);
-			xfree(s, l+1);
+			/* The unsplit_word is the original word; if its been split
+			 * into stem+suffix, and either one hasn't been choosen, then
+			 * neither should be printed.  Do, however, put brackets around
+			 * the original word, and print that.
+			 */
+			t = sent->word[i].unsplit_word;
+			if (t)
+			{
+				l = strlen(t) + 2;
+				s = (char *) xalloc(l+1);
+				sprintf(s, "[%s]", t);
+				t = string_set_add(s, sent->string_set);
+				xfree(s, l+1);
+			}
+			else
+				t = string_set_add("", sent->string_set);
 		}
 		else if (opts->display_word_subscripts)
 		{
 			t = pi->chosen_disjuncts[i]->string;        
 			/* get rid of those ugly ".Ixx" */
 			if (is_idiom_word(t)) {
-				l = strlen(t);
-				s = (char *) xalloc(l+1);
-				strcpy(s,t);
-				for (u=s; *u != '.'; u++)
-				  ;
+				s = strdup(t);
+				u = strrchr(s, '.');
 				*u = '\0';
 				t = string_set_add(s, sent->string_set);
-				xfree(s, l+1);
+				free(s);
+			}
+
+			/* Suppress the empty word. */
+			if (0 == strcmp(t, EMPTY_WORD))
+			{
+				t = string_set_add("", sent->string_set);
+			}
+
+			/* Concatenate the stem and the sufix together into one word */
+			if (HIDE_SUFFIX)
+			{
+				if (0 == strncmp(t, SUFFIX_WORD, SUFFIX_WORD_L) &&
+				   pi->chosen_disjuncts[i-1])
+				{
+					const char * stem = pi->chosen_disjuncts[i-1]->string;
+					size_t len = strlen(stem) + strlen (t);
+					char * join = malloc(len+1);
+					strcpy(join, stem);
+					u = strrchr(join, '.');
+					*u = '\0';
+					strcat(join, t + SUFFIX_WORD_L);
+					t = string_set_add(join, sent->string_set);
+					free(join);
+				}
+
+				/* Suppress printing of the stem, if the next word is the suffix */
+				if ((i+1 < sent->length) &&
+				    (pi->chosen_disjuncts[i+1]))
+				{
+					const char * next = pi->chosen_disjuncts[i+1]->string;
+					if (0 == strncmp(next, SUFFIX_WORD, SUFFIX_WORD_L) &&
+					    0 != strcmp(next, EMPTY_WORD))
+					{
+						t = string_set_add("", sent->string_set);
+					}
+				}
 			}
 		}
 		else
 		{
 			/* XXX This is wrong, since it fails to indicate what
 			 * was actally used for the parse, which might not actually 
-			 * be alternative 0.  We should use linkage->->word[i] instead,
-			 * and, umm, manually strip the subscript, or something ...
+			 * be alternative 0.  We should do like the above, and then
+			 * manually strip the subscript.
+			 * Except that this code is never ever reached, because 
+			 * opts->display_word_subscripts is always true...
 			 */
 			t = sent->word[i].alternatives[0];
 		}
@@ -396,8 +479,8 @@ void compute_chosen_words(Sentence sent, Linkage linkage)
 /** 
  * Print the indicated linkage into a utf8-diagram.
  * Works fine for general utf8 multi-byte sentences.
- * links and connectors are still mostly assumed to be ASCII, though;
- * to fix this, grep for strlen in he code below.
+ * Links and connectors are still mostly assumed to be ASCII, though;
+ * to fix this, grep for "strlen" in the code below, replace by utf8 len.
  *
  * Returned string is allocated with exalloc.  
  * Needs to be freed with linkage_free_diagram()
@@ -407,11 +490,16 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 	int i, j, k, cl, cr, row, top_row;
 	const char *s;
 	char *t;
-	int print_word_0 = 0, print_word_N = 0, N_wall_connectors, suppressor_used;
+	Boolean print_word_0 = 0, print_word_N = 0;
+	int N_wall_connectors, suppressor_used;
 	int center[MAX_SENTENCE];
 	char connector[MAX_TOKEN_LENGTH];
 	int line_len, link_length;
+#ifdef USE_FAT_LINKAGES
 	Sublinkage *sublinkage=&(linkage->sublinkage[linkage->current]);
+#else
+	Sublinkage *sublinkage=&(linkage->sublinkage);
+#endif /* USE_FAT_LINKAGES */
 	int N_links = sublinkage->num_links;
 	Link **ppla = sublinkage->link;
 	String * string;
@@ -425,9 +513,11 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 	char xpicture[MAX_HEIGHT][MAX_LINE];
 	size_t start[MAX_HEIGHT];
 
+	Boolean display_suffixes = opts->display_suffixes;
 
 	string = string_new();
 
+	/* Do we want to print the left wall? */
 	N_wall_connectors = 0;
 	if (dict->left_wall_defined)
 	{
@@ -440,7 +530,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 				{
 					if (ppla[j]->r == linkage->num_words-1) continue;
 					N_wall_connectors ++;
-					if (strcmp(ppla[j]->lc->string, LEFT_WALL_SUPPRESS)==0)
+					if (0 == strcmp(ppla[j]->lc->string, LEFT_WALL_SUPPRESS))
 					{
 						suppressor_used = TRUE;
 					}
@@ -455,6 +545,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 		print_word_0 = TRUE;
 	}
 
+	/* Do we want to print the right wall? */
 	N_wall_connectors = 0;
 	if (dict->right_wall_defined)
 	{
@@ -464,7 +555,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 			if (ppla[j]->r == linkage->num_words-1)
 			{
 				N_wall_connectors ++;
-				if (strcmp(ppla[j]->lc->string, RIGHT_WALL_SUPPRESS)==0)
+				if (0 == strcmp(ppla[j]->lc->string, RIGHT_WALL_SUPPRESS))
 				{
 					suppressor_used = TRUE;
 				}
@@ -480,7 +571,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 
 	N_words_to_print = linkage->num_words;
 	if (!print_word_N) N_words_to_print--;
-	
+
 	set_centers(linkage, center, print_word_0, N_words_to_print);
 	line_len = center[N_words_to_print-1]+1;
 	
@@ -498,7 +589,15 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 			if (!print_word_0 && (ppla[j]->l == 0)) continue;
 			/* gets rid of the irrelevant link to the left wall */
 			if (!print_word_N && (ppla[j]->r == linkage->num_words-1)) continue;            
+			/* Get rid of links to empty words */
+			if (0 == strcmp(ppla[j]->name, EMPTY_WORD_SUPPRESS)) continue;
+
+			if (HIDE_SUFFIX &&
+			    0 == strncmp(ppla[j]->name, SUFFIX_SUPPRESS, SUFFIX_SUPPRESS_L))
+				continue;
+
 			/* gets rid of the irrelevant link to the right wall */
+			/* ??? */
 
 			/* put it into the lowest position */
 			cl = center[ppla[j]->l];
@@ -528,15 +627,18 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 			}
 			s = ppla[j]->name;
 			
+			/* display_link_subscripts is always true ... hard-coded */
 			if (opts->display_link_subscripts)
 			{
+#ifdef USE_FAT_LINKAGES
 				if (!is_utf8_alpha(s))
-				  s = "";
+				  s = "";  /* Don't print fat link connector name */
+#endif /* USE_FAT_LINKAGES */
 			}
 			else
 			{
 				if (!is_utf8_upper(s)) {
-				  s = "";   /* Don't print fat link connector name */
+				  s = "";
 				}
 			}
 			strncpy(connector, s, MAX_TOKEN_LENGTH-1);
@@ -596,7 +698,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 		for (row=0; row < top_row+1; row++) {
 			strcpy(xpicture[row+2],picture[row]);
 		}
-		top_row = top_row+2;
+		top_row += 2;
 	} else {
 		for (row=0; row < top_row+1; row++) {
 			strcpy(xpicture[2*row+2],picture[row]);
@@ -707,6 +809,10 @@ void linkage_free_senses(char * s)
 	exfree(s, strlen(s)+1);
 }
 
+/* Forward declarations, the gunk is at the bottom. */
+static const char * trailer(int mode);
+static const char * header(int mode);
+
 char * linkage_print_postscript(Linkage linkage, int mode)
 {
 	char * ps, * qs;
@@ -763,7 +869,7 @@ void print_expression_sizes(Sentence sent)
 	printf("\n\n");
 }
 
-const char * trailer(int mode)
+static const char * trailer(int mode)
 {
 	static const char * trailer_string=
 		"diagram\n"
@@ -775,7 +881,7 @@ const char * trailer(int mode)
 	else return "";
 }
 
-const char * header(int mode)
+static const char * header(int mode)
 {
 	static const char * header_string=
 		"%!PS-Adobe-2.0 EPSF-1.2\n"
