@@ -69,9 +69,9 @@ struct prune_context_s
 #ifdef USE_FAT_LINKAGES
 	char ** deletable;
 	char ** effective_dist;
+	int power_prune_mode;  /* either GENTLE or RUTHLESS */
 #endif /* USE_FAT_LINKAGES */
 	int power_cost;
-	int power_prune_mode;  /* either GENTLE or RUTHLESS */
 	int N_changed;   /* counts the number of changes
 						   of c->word fields in a pass */
 
@@ -134,13 +134,14 @@ static inline int hash_S(Connector * c)
 }
 
 /**
- * This is almost identical to match().  Its reason for existance
+ * This is almost identical to do_match().  Its reason for existance
  * is the rather subtle fact that with "and" can transform a "Ss"
  * connector into "Sp".  This means that in order for pruning to
  * work, we must allow a "Ss" connector on word match an "Sp" connector
  * on a word to its right.  This is what this version of match allows.
  * We assume that a is on a word to the left of b.
  */
+#ifdef USE_FAT_LINKAGES
 int prune_match(int dist, Connector *a, Connector *b)
 {
 	const char *s, *t;
@@ -148,14 +149,10 @@ int prune_match(int dist, Connector *a, Connector *b)
 
 	if (a->label != b->label) return FALSE;
 
-	x = hash_S(a);
-	y = hash_S(b);
-	if (x != y) return FALSE;
-
 	s = a->string;
 	t = b->string;
 
-	while(s < a->prune_string || t < b->prune_string)
+	while(isupper((int)*s) || isupper((int)*t))
 	{
 		if (*s != *t) return FALSE;
 		s++;
@@ -163,7 +160,7 @@ int prune_match(int dist, Connector *a, Connector *b)
 	}
 
 	/*	printf("PM: a=%4s b=%4s  ap=%d bp=%d  a->ll=%d b->ll=%d  dist=%d\n",
-		   s, t, x, y, a->length_limit, b->length_limit, dist); */
+	   s, t, x, y, a->length_limit, b->length_limit, dist); */
 	if (dist > a->length_limit || dist > b->length_limit) return FALSE;
 
 	x = a->priority;
@@ -201,10 +198,10 @@ int prune_match(int dist, Connector *a, Connector *b)
 		{
 			if ((*s == '*') || (*t == '*') ||
 				((*s == *t) && (*s != '^')))
-			{
 			  /* this last case here is rather obscure.  It prevents
 				 '^' from matching '^'.....Is this necessary?
 					 ......yes, I think it is.   */
+			{
 				s++;
 				t++;
 			}
@@ -247,6 +244,38 @@ int prune_match(int dist, Connector *a, Connector *b)
 		return FALSE;
 }
 
+#else /* not USE_FAT_LINKAGES */
+
+int prune_match(int dist, Connector *a, Connector *b)
+{
+	const char *s, *t;
+
+	s = a->string;
+	t = b->string;
+
+	while(isupper((int)*s) || isupper((int)*t))
+	{
+		if (*s != *t) return FALSE;
+		s++;
+		t++;
+	}
+
+	if (dist > a->length_limit || dist > b->length_limit) return FALSE;
+
+	while ((*s != '\0') && (*t != '\0'))
+	{
+		if ((*s == '*') || (*t == '*') || (*s == *t))
+		{
+			s++;
+			t++;
+		}
+		else
+			return FALSE;
+	}
+	return TRUE;
+}
+#endif /* USE_FAT_LINKAGES */
+
 static void zero_connector_table(connector_table *ct)
 {
 	memset(ct, 0, sizeof(Connector *) * CONTABSZ);
@@ -265,10 +294,15 @@ static void insert_connector(connector_table *ct, Connector * c)
 
 	for (e = ct[h]; e != NULL; e = e->tableNext)
 	{
+#ifdef USE_FAT_LINKAGES
 		if ((strcmp(c->string, e->string) == 0) && 
-		    (c->label == e->label) && 
-		    (c->priority == e->priority))
+		    (c->priority == e->priority) &&
+		    (c->label == e->label))
 			return;
+#else /* USE_FAT_LINKAGES */
+		if (strcmp(c->string, e->string) == 0)
+			return;
+#endif /* USE_FAT_LINKAGES */
 	}
 	c->tableNext = ct[h];
 	ct[h] = c;
@@ -413,7 +447,7 @@ void prune(Sentence sent)
    another.  It works by hashing them all, and checking for domination.
 */
 
-#if FALSE
+#if 0
 /* ============================================================x */
 
 /*
@@ -829,8 +863,10 @@ static int mark_dead_connectors(connector_table *ct, Exp * e, int dir)
 		{
 			Connector dummy;
 			init_connector(&dummy);
+#ifdef USE_FAT_LINKAGES
 			dummy.label = NORMAL_LABEL;
 			dummy.priority = THIN_priority;
+#endif /* USE_FAT_LINKAGES */
 			dummy.string = e->u.string;
 			if (!matches_S(ct, &dummy, dir))
 			{
@@ -905,6 +941,9 @@ static void clean_up_expressions(Sentence sent, int w)
 	sent->word[w].x = head_node.next;
 }
 
+/* #define DBG(X) X */
+#define DBG(X) 
+
 void expression_prune(Sentence sent)
 {
 	int N_deleted;
@@ -926,15 +965,15 @@ void expression_prune(Sentence sent)
 			/* For every expression in word */
 			for (x = sent->word[w].x; x != NULL; x = x->next)
 			{
-/* printf("before marking: "); print_expression(x->exp); printf("\n"); */
+DBG(printf("before marking: "); print_expression(x->exp); printf("\n"););
 				N_deleted += mark_dead_connectors(ct, x->exp, '-');
-/* printf(" after marking: "); print_expression(x->exp); printf("\n"); */
+DBG(printf(" after marking: "); print_expression(x->exp); printf("\n"););
 			}
 			for (x = sent->word[w].x; x != NULL; x = x->next)
 			{
-/* printf("before purging: "); print_expression(x->exp); printf("\n"); */
+DBG(printf("before purging: "); print_expression(x->exp); printf("\n"););
 				x->exp = purge_Exp(x->exp);
-/* printf("after purging: "); print_expression(x->exp); printf("\n"); */
+DBG(printf("after purging: "); print_expression(x->exp); printf("\n"););
 			}
 
 			/* gets rid of X_nodes with NULL exp */
@@ -1006,9 +1045,9 @@ void expression_prune(Sentence sent)
 	1) successive connectors on the same disjunct have to go to
 	   nearer and nearer words.
 
-	2) two deep connectors cannot attach to eachother
-	   (A connectors is deep if it is not the first in its list, it
-	   is shallow if it is the first in its list, it is deepest if it
+	2) two deep connectors cannot attach to each other
+	   (A connectors is deep if it is not the first in its list; it
+	   is shallow if it is the first in its list; it is deepest if it
 	   is the last on its list.)
 
 	3) on two adjacent words, a pair of connectors can be used
@@ -1144,7 +1183,7 @@ static power_table * power_table_new(Sentence sent)
 	pt = (power_table *) malloc (sizeof(power_table));
 	pt->power_table_size = sent->length;
 
-   /* first we initialize the word fields of the connectors, and
+   /* First, we initialize the word fields of the connectors, and
 	  eliminate those disjuncts with illegal connectors */
 	for (w=0; w<sent->length; w++)
 	{
@@ -1210,9 +1249,9 @@ static void clean_table(int size, C_list ** t)
 {
 	int i;
 	C_list * m, * xm, * head;
-	for (i=0; i<size; i++) {
+	for (i = 0; i < size; i++) {
 		head = NULL;
-		for (m=t[i]; m!=NULL; m=xm) {
+		for (m = t[i]; m != NULL; m = xm) {
 			xm = m->next;
 			if (m->c->word != BAD_WORD) {
 				m->next = head;
@@ -1230,6 +1269,7 @@ static void clean_table(int size, C_list ** t)
  * (and the two words that these came from) and returns TRUE if it is
  * possible for these two to match based on local considerations.
  */
+#ifdef USE_FAT_LINKAGES
 static int possible_connection(prune_context *pc,
                                Connector *lc, Connector *rc,
                                int lshallow, int rshallow,
@@ -1243,7 +1283,7 @@ static int possible_connection(prune_context *pc,
 	assert(lword < rword, "Bad word order in possible connection.");
 
 	/* Now, notice that the only differences between the following two
-	   cases is that (1) ruthless uses match and gentle uses prune_match.
+	   cases is that (1) ruthless uses do_match and gentle uses prune_match.
 	   and (2) ruthless doesn't use deletable[][].  This latter fact is
 	   irrelevant, since deletable[][] is now guaranteed to have been
 	   created. */
@@ -1264,28 +1304,52 @@ static int possible_connection(prune_context *pc,
 		} else {
 			if ((!pc->null_links) &&
 				(lc->next == NULL) && (rc->next == NULL) && (!lc->multi) && (!rc->multi)
-#ifdef USE_FAT_LINKAGES
 				&& !pc->deletable[lword][rword]
-#endif /* USE_FAT_LINKAGES */
 				)
 			{
 				return FALSE;
 			}
 		}
-#ifdef USE_FAT_LINKAGES
 		return prune_match(pc->effective_dist[lword][rword], lc, rc);
-#else
-		return prune_match(rword - lword, lc, rc);
-#endif /* USE_FAT_LINKAGES */
 	}
 }
+#else /* not USE_FAT_LINKAGES */
+
+static int possible_connection(prune_context *pc,
+                               Connector *lc, Connector *rc,
+                               int lshallow, int rshallow,
+                               int lword, int rword)
+{
+	if ((!lshallow) && (!rshallow)) return FALSE;
+	  /* two deep connectors can't work */
+	if ((lc->word > rword) || (rc->word < lword)) return FALSE;
+	  /* word range constraints */
+
+	assert(lword < rword, "Bad word order in possible connection.");
+
+	if (lword == rword-1) {
+		if (!((lc->next == NULL) && (rc->next == NULL))) return FALSE;
+	}
+	else
+	if ((!pc->null_links) &&
+	    (lc->next == NULL) &&
+	    (rc->next == NULL) &&
+	    (!lc->multi) && (!rc->multi))
+	{
+		return FALSE;
+	}
+	return prune_match(rword - lword, lc, rc);
+}
+#endif /* USE_FAT_LINKAGES */
 
 
 /**
  * This returns TRUE if the right table of word w contains
  * a connector that can match to c.  shallow tells if c is shallow.
  */
-static int right_table_search(prune_context *pc, int w, Connector *c, int shallow, int word_c)
+static int 
+right_table_search(prune_context *pc, int w, Connector *c,
+                   int shallow, int word_c)
 {
 	int size, h;
 	C_list *cl;
@@ -1308,7 +1372,9 @@ static int right_table_search(prune_context *pc, int w, Connector *c, int shallo
  * This returns TRUE if the right table of word w contains
  * a connector that can match to c.  shallows tells if c is shallow
  */
-static int left_table_search(prune_context *pc, int w, Connector *c, int shallow, int word_c)
+static int 
+left_table_search(prune_context *pc, int w, Connector *c,
+                  int shallow, int word_c)
 {
 	int size, h;
 	C_list *cl;
@@ -1319,45 +1385,34 @@ static int left_table_search(prune_context *pc, int w, Connector *c, int shallow
 	h = connector_hash(c) & (size-1);
 	for (cl = pt->l_table[w][h]; cl != NULL; cl = cl->next)
 	{
-	  if (possible_connection(pc, c, cl->c, shallow, cl->shallow, word_c, w))
-		{
-		  return TRUE;
-	  }
+		if (possible_connection(pc, c, cl->c, shallow, cl->shallow, word_c, w))
+			return TRUE;
 	}
 	return FALSE;
 }
 
-#if NOT_USED_NOW
-static int ok_cwords(Sentence sent, Connector *c)
-{
-	for (; c != NULL; c=c->next) {
-		if (c->word == BAD_WORD) return FALSE;
-		if (c->word >= sent->length) return FALSE;
-	}
-	return TRUE;
-}
-#endif
-
 /**
- * take this connector list, and try to match it with the words
- * w-1, w-2, w-3...Returns the word to which the first connector of the
- * list could possibly be matched.  If c is NULL, returns w.  If there
- * is no way to match this list, it returns a negative number.
+ * Take this connector list, and try to match it with the words
+ * w-1, w-2, w-3...  Returns the word to which the first connector of
+ * the list could possibly be matched.  If c is NULL, returns w.  If 
+ * there is no way to match this list, it returns a negative number.
  * If it does find a way to match it, it updates the c->word fields
  * correctly.
  */
-static int left_connector_list_update(prune_context *pc, Connector *c, int word_c, int w, int shallow)
+static int
+left_connector_list_update(prune_context *pc, Connector *c,
+                           int word_c, int w, int shallow)
 {
 	int n;
 	int foundmatch;
 
-	if (c==NULL) return w;
+	if (c == NULL) return w;
 	n = left_connector_list_update(pc, c->next, word_c, w, FALSE) - 1;
 	if (((int) c->word) < n) n = c->word;
 
 	/* n is now the rightmost word we need to check */
 	foundmatch = FALSE;
-	for (; (n >= 0) && ((w-n) < MAX_SENTENCE); n--) {
+	for (; n >= 0 ; n--) {
 		pc->power_cost++;
 		if (right_table_search(pc, n, c, shallow, word_c)) {
 			foundmatch = TRUE;
@@ -1372,26 +1427,27 @@ static int left_connector_list_update(prune_context *pc, Connector *c, int word_
 }
 
 /**
- * take this connector list, and try to match it with the words
- * w+1, w+2, w+3...Returns the word to which the first connector of the
- * list could possibly be matched.  If c is NULL, returns w.  If there
- * is no way to match this list, it returns a number greater than N_words-1
- * If it does find a way to match it, it updates the c->word fields
- * correctly.
+ * Take this connector list, and try to match it with the words
+ * w+1, w+2, w+3...  Returns the word to which the first connector of
+ * the list could possibly be matched.  If c is NULL, returns w.  If
+ * there is no way to match this list, it returns a number greater than
+ * N_words - 1.   If it does find a way to match it, it updates the
+ * c->word fields correctly.
  */
-static int right_connector_list_update(prune_context *pc, Sentence sent, Connector *c,
-                                       int word_c, int w, int shallow)
+static int
+right_connector_list_update(prune_context *pc, Sentence sent, Connector *c,
+                            int word_c, int w, int shallow)
 {
 	int n;
 	int foundmatch;
 
-	if (c==NULL) return w;
+	if (c == NULL) return w;
 	n = right_connector_list_update(pc, sent, c->next, word_c, w, FALSE) + 1;
 	if (c->word > n) n = c->word;
 
 	/* n is now the leftmost word we need to check */
 	foundmatch = FALSE;
-	for (; (n < sent->length) && ((n-w) < MAX_SENTENCE); n++) {
+	for (; n < sent->length ; n++) {
 		pc->power_cost++;
 		if (left_table_search(pc, n, c, shallow, word_c)) {
 			foundmatch = TRUE;
@@ -1416,12 +1472,12 @@ int power_prune(Sentence sent, int mode, Parse_Options opts)
 
 	pc = (prune_context *) malloc (sizeof(prune_context));
 	pc->power_cost = 0;
-	pc->power_prune_mode = mode;
 	pc->null_links = (opts->min_null_count > 0);
 	pc->N_changed = 1;  /* forces it always to make at least two passes */
 
 	pc->sent = sent;
 #ifdef USE_FAT_LINKAGES
+	pc->power_prune_mode = mode;
 	pc->deletable = sent->deletable;
 	pc->effective_dist = sent->effective_dist;
 	count_set_effective_distance(sent);
@@ -1443,8 +1499,8 @@ int power_prune(Sentence sent, int mode, Parse_Options opts)
 			for (d = sent->word[w].d; d != NULL; d = d->next) {
 				if (d->left == NULL) continue;
 				if (left_connector_list_update(pc, d->left, w, w, TRUE) < 0) {
-					for (c=d->left  ;c!=NULL; c = c->next) c->word = BAD_WORD;
-					for (c=d->right ;c!=NULL; c = c->next) c->word = BAD_WORD;
+					for (c=d->left;  c != NULL; c = c->next) c->word = BAD_WORD;
+					for (c=d->right; c != NULL; c = c->next) c->word = BAD_WORD;
 					N_deleted++;
 					total_deleted++;
 				}
@@ -1465,7 +1521,7 @@ int power_prune(Sentence sent, int mode, Parse_Options opts)
 			sent->word[w].d = nd;
 		}
 		if (verbosity > 2) {
-		   printf("l->r pass changed %d and deleted %d\n",pc->N_changed,N_deleted);
+			printf("l->r pass changed %d and deleted %d\n",pc->N_changed,N_deleted);
 		}
 
 		if (pc->N_changed == 0) break;
@@ -1477,9 +1533,9 @@ int power_prune(Sentence sent, int mode, Parse_Options opts)
 			if (parse_options_resources_exhausted(opts)) break;
 			for (d = sent->word[w].d; d != NULL; d = d->next) {
 				if (d->right == NULL) continue;
-				if (right_connector_list_update(pc, sent, d->right,w,w,TRUE) >= sent->length){
-					for (c=d->right;c!=NULL; c = c->next) c->word = BAD_WORD;
-					for (c=d->left ;c!=NULL; c = c->next) c->word = BAD_WORD;
+				if (right_connector_list_update(pc, sent, d->right, w, w, TRUE) >= sent->length) {
+					for (c=d->right; c != NULL; c = c->next) c->word = BAD_WORD;
+					for (c=d->left;  c != NULL; c = c->next) c->word = BAD_WORD;
 					N_deleted++;
 					total_deleted++;
 				}
@@ -1500,7 +1556,7 @@ int power_prune(Sentence sent, int mode, Parse_Options opts)
 		}
 
 		if (verbosity > 2) {
-		   printf("r->l pass changed %d and deleted %d\n", pc->N_changed,N_deleted);
+			printf("r->l pass changed %d and deleted %d\n", pc->N_changed,N_deleted);
 		}
 
 		if (pc->N_changed == 0) break;
@@ -1839,7 +1895,7 @@ static int pp_prune(Sentence sent, Parse_Options opts)
 		}
 
 		if (verbosity > 2) {
-		   printf("pp_prune pass deleted %d\n", N_deleted);
+			printf("pp_prune pass deleted %d\n", N_deleted);
 		}
 
 	}
