@@ -1,6 +1,7 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* Copyright 2013 Linas Vepstas                                          */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
@@ -407,7 +408,7 @@ static int check_connector(Dictionary dict, const char * s)
  * example, "make" < "make.n" < "make-up" -- suffixed words come after
  * the bare words, but before any other other words with non-ascii-alpha
  * characters (such as the hyphen in "make-up", or possibly UTF8
- * characters). Thus, stright "strcmp" can't be used to determine
+ * characters). Thus, plain "strcmp" can't be used to determine
  * dictionary order.
  *
  * Thus, a set of specialized string comparison and ordering functions
@@ -416,9 +417,11 @@ static int check_connector(Dictionary dict, const char * s)
  */
 /**
  * dict_order - order two dictionary words in proper sort order.
- * Return zero if the strings match, else return standard
- * (locale-dependent) UTF8 sort order.
- * XXX but its not UTF-8 ordered ...! Does this matter ???
+ * Return zero if the strings match, else return in a unique order.
+ * The order is NOT (locale-dependent) UTF8 sort order; its ordered
+ * baed on numeric values single bytes.  I beleive that this will
+ * uniquely order UTF8 strings, just not in a LANG-dependent
+ * (locale-dependent) order.
  */
 /* verbose version */
 /*
@@ -462,11 +465,13 @@ static inline int dict_order(const char *s, const char *t)
  * by "\0", and take the difference.  This behavior matches that
  * of the function dict_order().
  *
- * Note: wild-cards are not used in the English dictionaries;
- * nor are they used in the Persian or Arabic dictionaries...
- * perhaps they are used to handle stemming in some other
- * languages ?????? XXX FIXME, these wildcards might just be
- * old dead code ...
+ * Note: words in the dictionary itself don't have wild-card
+ * chars in them; the wild-card support is here only if you are
+ * searching for part of a word.
+ *
+ * XXX ... except that essentially no one is searching for a fraction
+ * of a word, and so this wild-card search is ... unused.  Mostly
+ * harmless, I suppose ...
  */
 static inline int dict_order_wild(const char * s, const char * t)
 {
@@ -476,7 +481,7 @@ static inline int dict_order_wild(const char * s, const char * t)
 	if ((*s == '*') || (*t == '*'))
 	{
 		/* Do not allow wild-card as the very first char.
-		 * Basically, allow *.v and *.eq in the Enlgish dict.
+		 * Basically, allow *.v and *.eq in the English dict.
 		 */
 		if (s != so) return 0;
 	}
@@ -493,7 +498,7 @@ static inline int dict_order_wild(const char * s, const char * t)
  * A subscript is the part that followes the last "." in the word, and
  * that does not begin with a digit.
  */
-static int dict_match(const char * s, const char * t)
+static Boolean dict_match(const char * s, const char * t)
 {
 	char *ds, *dt;
 	ds = strrchr(s, '.');
@@ -623,8 +628,8 @@ static Dict_node * rdictionary_lookup(Dict_node *llist,
  * dictionary_lookup_list() - return lookup list of words in the dictionary
  *
  * Returns a pointer to a lookup list of the words in the dictionary.
- * Matches include word that appear in idioms.  Use
- * abridged_lookup_list() to obtain matches, excluding idioms.
+ * Matches include words that appear in idioms.  To exclude idioms, use
+ * abridged_lookup_list() to obtain matches.
  *
  * This list is made up of Dict_nodes, linked by their right pointers.
  * The node, file and string fields are copied from the dictionary.
@@ -667,7 +672,7 @@ Boolean boolean_dictionary_lookup(Dictionary dict, const char *s)
 }
 
 /**
- * Return true if word is in dicationary, or if word is matched by
+ * Return true if word is in dictionary, or if word is matched by
  * regex.
  */
 Boolean find_word_in_dict(Dictionary dict, const char * word)
@@ -682,6 +687,91 @@ Boolean find_word_in_dict(Dictionary dict, const char * word)
 }
 
 
+/* ======================================================================== */
+/* Empty-word handling. */
+
+/* stems, by definition, always end with ".=" */
+#define STEM_MARK '='
+#define EMPTY_CONNECTOR "ZZZ"
+
+static Exp * make_optional_node(Dictionary dict, Exp * e);
+
+/**
+ * Insert empty-word connectors.
+ *
+ * Empty words are used to work around a fundamental parser design flaw.
+ * The flaw is that the parser assumes that there exist a fixed number of
+ * words in a sentence, independent of tokenization.  Unfortuntely, this
+ * is not true when correcting spelling errors, or when the dictionary
+ * contains entries for plain words and also as stems.  For example,
+ * in the Russian dictionary, это.msi appears as a single word, but can
+ * also be split into эт.= =о.mnsi. The problem arises because это.msi
+ * is a single word, while эт.= =о.mnsi counts as two words, and there 
+ * is no pretty way to handle both during parsing.  Thus a work-around
+ * is introduced: add the empty wored =.zzz: ZZZ+; to the dictionary.
+ * this becomes a pseudo-suffix that can attach to any plain word.  It
+ * can attach to any plain word only because the routine below,
+ * add_empty_word(), adds the corresponding connector ZZZ- to the plain
+ * word.  This is don "on the fly", because we don't want to pollute the
+ * dictionary with this stuff. Besides, the Russian dictionary has
+ * more then 23K words that qualify for this treatment (It has 22.5K
+ * words that appear both as plain words, and as stems, and can thus
+ * attach to null suffixes. For non-null suffix splits, there are
+ * clearly many more.)
+ *
+ * Note that the printing of ZZZ connectors is supressed in print.c,
+ * although API users will see this link.
+ */
+
+void add_empty_word(Dictionary dict, Dict_node * dn)
+{
+	size_t len;
+	Exp *zn, *an;
+	E_list *elist, *flist;
+
+	if (! dict->empty_word_defined) return;
+
+	if (STEM_MARK == dn->string[0]) return;
+
+	len = strlen(dn->string);
+	if (STEM_MARK == dn->string[len-1]) return;
+	if (0 == strcmp(dn->string, LEFT_WALL_WORD)) return;
+	if (0 == strcmp(dn->string, RIGHT_WALL_WORD)) return;
+
+	/* If we are here, then this appears to be not a stem, not a
+	 * suffix, and not an idiom word.
+	 * Create {ZZZ+} & (plain-word-exp). */
+
+	/* zn points at {ZZZ+} */
+	zn = Exp_create(dict);
+	zn->dir = '+';
+	zn->u.string = string_set_add(EMPTY_CONNECTOR, dict->string_set);
+	zn->multi = FALSE;
+	zn->type = CONNECTOR_type;
+	zn->cost = 0.0f;
+	zn = make_optional_node(dict, zn);
+
+	/* flist is plain-word-exp */
+	flist = (E_list *) xalloc(sizeof(E_list));
+	flist->next = NULL;
+	flist->e = dn->exp;
+
+	/* elist is {ZZZ+} */
+	elist = (E_list *) xalloc(sizeof(E_list));
+	elist->next = flist;
+	elist->e = zn;
+
+	/* an will be {ZZZ+} & (plain-word-exp) */
+	an = Exp_create(dict);
+	an->type = AND_type;
+	an->cost = 0.0f;
+	an->u.l = elist;
+
+	/* replace the plain-word-exp with {ZZZ+} & (plain-word-exp) */
+	dn->exp = an;
+}
+
+
 /* ======================================================================== */
 /**
  * Allocate a new Exp node and link it into the exp_list for freeing later.
@@ -1254,7 +1344,7 @@ static Dict_node * dsw_vine_to_tree (Dict_node *root, int size)
  * and right fields of it are NULL.
  *
  * The resulting tree is highly unbalanced. It needs to be rebalanced
- * before used.
+ * before being used.  The DSW algo below is ideal for that.
  */
 Dict_node * insert_dict(Dictionary dict, Dict_node * n, Dict_node * newnode)
 {
@@ -1701,11 +1791,9 @@ Boolean read_dictionary(Dictionary dict)
  * Returns true if it finds a non-idiom dict_node in a file that matches
  * the string s.
  *
- ** note: this now DOES include non-file words in its search.
- *
  * Also sets parent and to_be_deleted appropriately.
  */
-static int find_one_non_idiom_node(Dict_node * p, Dict_node * dn,
+static Boolean find_one_non_idiom_node(Dict_node * p, Dict_node * dn,
                                    const char * s,
                                    Dict_node **parent, Dict_node **to_be_deleted)
 {
@@ -1914,7 +2002,7 @@ static void display_counts(Dict_node *dn)
 		printf("    ");
 		left_print_string(stdout, dn->string,
 			"                         ");
-		printf(" %5d  disjuncts ", len);
+		printf(" %5u  disjuncts ", len);
 		if (dn->file != NULL)
 		{
 			printf("<%s>", dn->file->file);
