@@ -38,6 +38,7 @@
 
 #define ENTITY_MARKER   "<marker-entity>"
 #define COMMON_ENTITY_MARKER   "<marker-common-entity>"
+#define INFIX_MARK '='
 
 /**
  * is_common_entity - Return true if word is a common noun or adjective
@@ -83,21 +84,29 @@ static Boolean is_proper_name(const char * word)
  * be a quotation mark. This works, because link-grammar is more
  * or less ignorant of quotes at this time.
  */
-static const wchar_t *list_of_quotes(void) 
+static const wchar_t* list_of_quotes(void) 
 {
+#ifdef _MSC_VER
+	/* Microsoft Visual Studio understands these unicode chars as
+	 * wide chars, so just return them as-is.  At least, this is what
+	 * Alexander Tkachuk reports, circa 17 April 2013.
+	 */
+	static const wchar_t* wqs = "\"«»《》【】『』`„“";
+#else
+	/* Unix (Linux) and GCC understand the embedded unicode chars
+	 * as utf8, so convert them to wide chars before use.
+	 */
 #define QUSZ 50
 	static wchar_t wqs[QUSZ];
 	mbstate_t mbs;
 	/* Single-quotes are used for abbreviations, don't mess with them */
 	/* const char * qs = "\"\'«»《》【】『』‘’`„“"; */
-	const char * qs = "\"«»《》【】『』`„“";
-
-	const char *pqs = qs;
+	const char* qs = "\"«»《》【】『』`„“";
 
+	const char* pqs = qs;
 	memset(&mbs, 0, sizeof(mbs));
-
 	mbsrtowcs(wqs, &pqs, QUSZ, &mbs);
-
+#endif /* _MSC_VER */
 	return wqs;
 }
 
@@ -162,7 +171,8 @@ static Boolean contains_digits(const char * s)
 /** 
  * Make the string 's' be the next word of the sentence. 
  * That is, it looks like 's' is a word we can handle, so record it
- * as a bona-fide word in the sentence.
+ * as a bona-fide word in the sentence.  Increment the sentence length
+ * when done.
  *
  * Do not issue the empty string.  
  */
@@ -182,6 +192,9 @@ static void issue_sentence_word(Sentence sent, const char * s, Boolean quote_fou
 	sent->word[len].alternatives[0] = string_set_add(s, sent->string_set);
 	sent->word[len].alternatives[1] = NULL;
 
+	/* Record the original form, as well. */
+	sent->word[len].unsplit_word = sent->word[len].alternatives[0];
+
 	/* Now we record whether the first character of the word is upper-case.
 	   (The first character may be made lower-case
 	   later, but we may want to get at the original version) */
@@ -231,7 +244,7 @@ static void add_suffix_alternatives(Tokenizer * tok,
 	 * happen to be the same as the suffix. Kind-of a weird hack,
 	 * but I'm not sure what else to do... */
 	sz = MIN(strlen(suffix) + 2, MAX_WORD);
-	buff[0] = '=';
+	buff[0] = INFIX_MARK;
 	strncpy(&buff[1], suffix, sz);
 	buff[sz] = 0;
 
@@ -295,6 +308,7 @@ static Boolean issue_alternatives(Sentence sent, Boolean quote_found)
 		sent->word = realloc(sent->word, (len+1) * sizeof(Word));
 		sent->word[len].x = NULL;
 		sent->word[len].d = NULL;
+		sent->word[len].unsplit_word = NULL;
 
 		sent->word[len].alternatives = tokenizer->pref_alternatives;
 		tokenizer->pref_alternatives = NULL;
@@ -317,6 +331,7 @@ static Boolean issue_alternatives(Sentence sent, Boolean quote_found)
 		sent->word = realloc(sent->word, (len+1) * sizeof(Word));
 		sent->word[len].x = NULL;
 		sent->word[len].d = NULL;
+		sent->word[len].unsplit_word = tokenizer->unsplit_word;
 
 		sent->word[len].alternatives = tokenizer->stem_alternatives;
 
@@ -343,6 +358,7 @@ static Boolean issue_alternatives(Sentence sent, Boolean quote_found)
 		sent->word = realloc(sent->word, (len+1) * sizeof(Word));
 		sent->word[len].x = NULL;
 		sent->word[len].d = NULL;
+		sent->word[len].unsplit_word = NULL;
 
 		sent->word[len].alternatives = tokenizer->suff_alternatives;
 		tokenizer->suff_alternatives = NULL;
@@ -429,6 +445,12 @@ static Boolean suffix_split(Tokenizer *tokenizer, Dictionary dict,
 	Dictionary afdict = dict->affix_table;
 	if (NULL == afdict) return FALSE;
 
+	/* Record the original word */
+	len = MIN(wend-w, MAX_WORD);
+	strncpy(newword, w, len);
+	newword[len] = '\0';
+	tokenizer->unsplit_word = string_set_add(newword, tokenizer->string_set);
+
 	p_strippable = afdict->p_strippable;
 	s_strippable = afdict->s_strippable;
 	prefix = afdict->prefix;
@@ -538,7 +560,7 @@ static void separate_word(Sentence sent, Parse_Options opts,
 	int  n_r_stripped = 0;
 	Boolean word_is_in_dict;
 	Boolean issued = FALSE;
-	Boolean have_empty_suffix = FALSE;
+	Boolean have_suffix = FALSE;
 
 	int found_number = 0;
 	int n_r_stripped_save;
@@ -555,7 +577,8 @@ static void separate_word(Sentence sent, Parse_Options opts,
 
 	if (dict->affix_table != NULL)
 	{
-		have_empty_suffix = dict->affix_table->have_empty_suffix;
+		have_suffix = (0 < dict->affix_table->s_strippable) ||
+		              (0 < dict->affix_table->p_strippable);
 	}
 
 	/* First, see if we can already recognize the word as-is. If
@@ -570,8 +593,18 @@ static void separate_word(Sentence sent, Parse_Options opts,
 
 	/* ... unless its a lang like Russian, which allows empty
 	 * suffixes, which have a real morphological linkage.
-	 * In which case, we have to try them out. */
-	if (word_is_in_dict & !have_empty_suffix)
+	 * In which case, we have to try them out.
+	 */
+	/* XXX FIXME: currently, we are doing a check for capitalized first
+	 * words in the build_sentence_expressions() routine, but really, 
+	 * we should be doing it here, and building two alternatives, one
+	 * capitalized, and one not.  This is particularly true for the
+	 * Russian dictionaries, where captialization and suffix splitting
+	 * only 'accidentally' work, due toe CAPITALIZED_WORDS in the regex
+	 * file, which matches capitalized stems (for all the wrong reasons...)
+	 * Now that we have alternatives, we should use them wisely.
+	 */
+	if (word_is_in_dict && !have_suffix)
 	{
 		issue_sentence_word(sent, word, quote_found);
 		return;
@@ -588,132 +621,132 @@ static void separate_word(Sentence sent, Parse_Options opts,
 		strip_left = afdict->strip_left;
 		strip_right = afdict->strip_right;
 		strip_units = afdict->strip_units;
-	}
 
-	/* If we found the word in the dict, but it also can have an
-	 * empty suffix, then just skip right over to suffix processing
-	 */
-	if (word_is_in_dict && have_empty_suffix)
-		goto do_suffix_processing;
-
-	/* Strip off punctuation, etc. on the left-hand side. */
-	/* XXX FIXME: this fails in certain cases: e.g.
-	 * "By the '50s, he was very prosperous."
-	 * where the leading quote is striped, and then "50s," cannot be
-	 * found in the dict.  Next, the comma is removed, and "50s" is still
-	 * not in the dict ... the trick was that the comma should be 
-	 * right-stripped first, then the possible quotes. 
-	 * More generally, the current implementation of the link-parser algo
-	 * does not support multiple alternative tokenizations; the viterbi
-	 * parser, under development, should be able to do better.
-	 */
-	for (;;)
-	{
-		for (i=0; i<l_strippable; i++)
-		{
-			/* This is UTF8-safe, I beleive ... */
-			sz = strlen(strip_left[i]);
-			if (strncmp(w, strip_left[i], sz) == 0)
-			{
-				issue_sentence_word(sent, strip_left[i], quote_found);
-				w += sz;
-				break;
-			}
-		}
-		if (i == l_strippable) break;
-	}
-
-	/* Its possible that the token consisted entirely of
-	 * left-punctuation, in which case, it has all been issued.
-	 * So -- we're done, return.
-	 */
-	if (w >= wend) return;
-
-	/* Now w points to the string starting just to the right of
-	 * any left-stripped characters.
-	 * stripped[] is an array of numbers, indicating the index
-	 * numbers (in the strip_right array) of any strings stripped off;
-	 * stripped[0] is the number of the first string stripped off, etc.
-	 * When it breaks out of this loop, n_stripped will be the number
-	 * of strings stripped off.
-	 */
-	for (n_r_stripped = 0; n_r_stripped < MAX_STRIP; n_r_stripped++) 
-	{
-		sz = MIN(wend-w, MAX_WORD);
-		strncpy(word, w, sz);
-		word[sz] = '\0';
-		if (wend == w) break;  /* it will work without this */
-
-		if (find_word_in_dict(dict, word))
-		{
-			word_is_in_dict = TRUE;
-			break;
-		}
-
-		for (i=0; i < r_strippable; i++)
-		{
-			len = strlen(strip_right[i]);
-
-			/* the remaining w is too short for a possible match */
-			if ((wend-w) < len) continue;
-			if (strncmp(wend-len, strip_right[i], len) == 0)
-			{
-				r_stripped[n_r_stripped] = strip_right[i];
-				wend -= len;
-				break;
-			}
-		}
-		if (i == r_strippable) break;
-	}
-
-	/* Is there a number in the word? If so, then search for
-	 * trailing units suffixes.
-	 */
-	if ((FALSE == word_is_in_dict) && contains_digits(word))
-	{
-		/* Same as above, but with a twist: the only thing that can
-		 * preceed a units suffix is a number. This is so that we can
-		 * split up things like "12ft" (twelve feet) but not split up
-		 * things like "Delft blue". Multiple passes allow for
-		 * constructions such as 12sq.ft.
+		/* If we found the word in the dict, but it also can have an
+		 * empty suffix, then just skip right over to suffix processing.
 		 */
-		n_r_stripped_save = n_r_stripped;
-		wend_save = wend;
-		for (; n_r_stripped < MAX_STRIP; n_r_stripped++) 
+		if (word_is_in_dict && have_suffix)
+			goto do_suffix_processing;
+
+		/* Strip off punctuation, etc. on the left-hand side. */
+		/* XXX FIXME: this fails in certain cases: e.g.
+		 * "By the '50s, he was very prosperous."
+		 * where the leading quote is striped, and then "50s," cannot be
+		 * found in the dict.  Next, the comma is removed, and "50s" is still
+		 * not in the dict ... the trick was that the comma should be 
+		 * right-stripped first, then the possible quotes. 
+		 * More generally, the current implementation of the link-parser algo
+		 * does not support multiple alternative tokenizations; the viterbi
+		 * parser, under development, should be able to do better.
+		 */
+		for (;;)
 		{
-			size_t sz = MIN(wend-w, MAX_WORD);
+			for (i=0; i<l_strippable; i++)
+			{
+				/* This is UTF8-safe, I beleive ... */
+				sz = strlen(strip_left[i]);
+				if (strncmp(w, strip_left[i], sz) == 0)
+				{
+					issue_sentence_word(sent, strip_left[i], quote_found);
+					w += sz;
+					break;
+				}
+			}
+			if (i == l_strippable) break;
+		}
+
+		/* Its possible that the token consisted entirely of
+		 * left-punctuation, in which case, it has all been issued.
+		 * So -- we're done, return.
+		 */
+		if (w >= wend) return;
+
+		/* Now w points to the string starting just to the right of
+		 * any left-stripped characters.
+		 * stripped[] is an array of numbers, indicating the index
+		 * numbers (in the strip_right array) of any strings stripped off;
+		 * stripped[0] is the number of the first string stripped off, etc.
+		 * When it breaks out of this loop, n_stripped will be the number
+		 * of strings stripped off.
+		 */
+		for (n_r_stripped = 0; n_r_stripped < MAX_STRIP; n_r_stripped++) 
+		{
+			sz = MIN(wend-w, MAX_WORD);
 			strncpy(word, w, sz);
 			word[sz] = '\0';
 			if (wend == w) break;  /* it will work without this */
 
-			/* Number */
-			if (is_number(word))
+			if (find_word_in_dict(dict, word))
 			{
-				found_number = 1;
+				word_is_in_dict = TRUE;
 				break;
 			}
 
-			for (i=0; i < u_strippable; i++)
+			for (i=0; i < r_strippable; i++)
 			{
-				len = strlen(strip_units[i]);
+				len = strlen(strip_right[i]);
 
 				/* the remaining w is too short for a possible match */
 				if ((wend-w) < len) continue;
-				if (strncmp(wend-len, strip_units[i], len) == 0)
+				if (strncmp(wend-len, strip_right[i], len) == 0)
 				{
-					r_stripped[n_r_stripped] = strip_units[i];
+					r_stripped[n_r_stripped] = strip_right[i];
 					wend -= len;
 					break;
 				}
 			}
-			if (i == u_strippable) break;
+			if (i == r_strippable) break;
 		}
 
-		/* The root *must* be a number! */
-		if (0 == found_number)
+		/* Is there a number in the word? If so, then search for
+		 * trailing units suffixes.
+		 */
+		if ((FALSE == word_is_in_dict) && contains_digits(word))
 		{
-			wend = wend_save;
-			n_r_stripped = n_r_stripped_save;
+			/* Same as above, but with a twist: the only thing that can
+			 * preceed a units suffix is a number. This is so that we can
+			 * split up things like "12ft" (twelve feet) but not split up
+			 * things like "Delft blue". Multiple passes allow for
+			 * constructions such as 12sq.ft.
+			 */
+			n_r_stripped_save = n_r_stripped;
+			wend_save = wend;
+			for (; n_r_stripped < MAX_STRIP; n_r_stripped++) 
+			{
+				size_t sz = MIN(wend-w, MAX_WORD);
+				strncpy(word, w, sz);
+				word[sz] = '\0';
+				if (wend == w) break;  /* it will work without this */
+
+				/* Number */
+				if (is_number(word))
+				{
+					found_number = 1;
+					break;
+				}
+
+				for (i=0; i < u_strippable; i++)
+				{
+					len = strlen(strip_units[i]);
+
+					/* the remaining w is too short for a possible match */
+					if ((wend-w) < len) continue;
+					if (strncmp(wend-len, strip_units[i], len) == 0)
+					{
+						r_stripped[n_r_stripped] = strip_units[i];
+						wend -= len;
+						break;
+					}
+				}
+				if (i == u_strippable) break;
+			}
+
+			/* The root *must* be a number! */
+			if (0 == found_number)
+			{
+				wend = wend_save;
+				n_r_stripped = n_r_stripped_save;
+			}
 		}
 	}
 
@@ -731,7 +764,7 @@ i		 * "wend" to the end of the word. */
 
 do_suffix_processing:
 	/* OK, now try to strip suffixes. */
-	if (!word_is_in_dict || have_empty_suffix)
+	if (!word_is_in_dict || have_suffix)
 	{
 		Tokenizer *tokenizer = &sent->tokenizer;
 
@@ -740,7 +773,14 @@ do_suffix_processing:
 		tokenizer->stem_alternatives = NULL;
 		tokenizer->suff_alternatives = NULL;
 
-		word_is_in_dict = suffix_split(tokenizer, dict, w, wend);
+		/* If the word is in the dict and can also be split into two,
+		 * then we need to insert a two-word version, with the second
+		 * word being the empty word.  This is a crazy hack to always
+		 * keep a uniform word-count */
+		if (word_is_in_dict)
+			add_suffix_alternatives(tokenizer, word, "");
+
+		word_is_in_dict |= suffix_split(tokenizer, dict, w, wend);
 	}
 
 	/* word is now what remains after all the stripping has been done */
@@ -1041,7 +1081,7 @@ static X_node * guess_misspelled_word(Sentence sent, int i, const char * s)
  * as two words, not one. But the former only issues as one word.  So,
  * should we issue two word or one?  ... and so we can't correctly
  * build sentence expressions for this, since we don't have the correct
- * word count bythis point.  The viterbi incremental parser won't have
+ * word count by this point.  The viterbi incremental parser won't have
  * this bug.
  */
 void build_sentence_expressions(Sentence sent, Parse_Options opts)
@@ -1103,6 +1143,15 @@ void build_sentence_expressions(Sentence sent, Parse_Options opts)
 			 *
 			 * XXX This rule is English-language-oriented, and should be
 			 * abstracted.
+			 *
+			 * XXX For the first-word case, we should be handling capitalization
+			 * as an alternative, when doing separate_word(), and not here.
+			 * separate_word() should build capitalized and non-capitalized
+			 * altenatives.  This is especially true for Russian, where we
+			 * need to deal with capitalized stems; thhis is not really the
+			 * right place to do it, and this works 'by accident' only because
+			 * there is a CAPITALIZED_WORDS regex match for Russian that matches
+			 * stems. Baaaddd.
 			 */
 			if ((i == first_word ||
 			     (i > 0 && strcmp(":", sent->word[i-1].alternatives[0])==0) || 
@@ -1158,7 +1207,7 @@ void build_sentence_expressions(Sentence sent, Parse_Options opts)
 						}
 	
 						/* If we are here, then we want both upper and lower case
-						 * expressions. The upper-case ones were build above, so now
+						 * expressions. The upper-case ones were built above, so now
 						 * append the lower-case ones. */
 						e = build_word_expressions(sent->dict, lc);
 						we = catenate_X_nodes(we, e);
