@@ -44,6 +44,7 @@
 
 #ifdef _WIN32
 #include <windows.h>
+#include <mbctype.h>
 
 #ifdef _MSC_VER
 /* The Microsoft Visual C compiler doesn't support the "inline" keyword. */
@@ -61,6 +62,8 @@
 
 /* MS Visual C does not support some C99 standard floating-point functions */
 #define fmaxf(a,b) ((a) > (b) ? (a) : (b))
+#define floorf(x) floor(x)
+#define ceilf(x) ceil(x)
 
 #endif /* _MSC_VER */
 
@@ -88,12 +91,15 @@ char * strndup (const char *str, size_t size);
 #endif /* _MSC_VER || __MINGW32__ */
 
 /*
- * CYGWIN on Windows doesn't have UTF8 support, or wide chars ... 
- * However, MS Visual C appearently does, as does MinGW.  Since
+ * CYGWIN prior to version 1.7 did not have UTF8 support, or wide
+ * chars ... However, MS Visual C does, as does MinGW.  Since
  * some users have both cygwin and MinGW installed, crap out the 
- * UTF8 code only when MinGW is missing.
+ * UTF8 code only when MinGW is missing (and the CYGWIN version
+ * is very old) XXX This code is dangerous and should be removed.
  */
 #if defined (__CYGWIN__) && !defined(__MINGW32__)
+#if CYGWIN_VERSION_DLL_MAJOR < 1007
+#error "Your Cygwin version is too old! Version 1.7 or later is needed for UTF8 support!"
 #define mbstate_t char
 #define mbrtowc(w,s,n,x)  ({*((char *)(w)) = *(s); 1;})
 #define wcrtomb(s,w,x)    ({*((char *)(s)) = ((char)(w)); 1;})
@@ -107,6 +113,7 @@ char * strndup (const char *str, size_t size);
 #define WEOF      EOF
 #define towlower  tolower
 #define towupper  toupper
+#endif /* CYGWIN_VERSION_DLL_MAJOR < 1007 */
 #endif /* __CYGWIN__ and not __MINGW32__ */
 
 #endif /* _WIN32 */
@@ -269,8 +276,6 @@ void safe_strcpy(char *u, const char * v, size_t usize);
 void safe_strcat(char *u, const char *v, size_t usize);
 char *safe_strdup(const char *u);
 
-void left_print_string(FILE* fp, const char *, const char *);
-
 /* routines for allocating basic objects */
 void init_memusage(void);
 void * xalloc(size_t) GNUC_MALLOC;
@@ -306,7 +311,7 @@ wchar_t * get_file_contents(const char *filename);
 static inline int next_power_of_two_up(int i)
 {
    int j=1;
-   while(j<i) j = j<<1;
+   while(j<i) j <<= 1;
    return j;
 }
 
