@@ -52,6 +52,7 @@ static struct
 	int display_walls;
 	int display_disjuncts;
 	int display_senses;
+	int display_suffixes;
 } local;
 
 typedef enum
@@ -94,6 +95,7 @@ static Switch default_switches[] =
 #if defined HAVE_HUNSPELL || defined HAVE_ASPELL
    {"spell",      Bool, "Use spell-guesser for unknown words",  &local.spell_guess},
 #endif /* HAVE_HUNSPELL */
+   {"suffixes",   Bool, "Display of word stems+suffixes",  &local.display_suffixes},
    {"timeout",    Int,  "Abort parsing after this many seconds", &local.timeout},
 #ifdef USE_FAT_LINKAGES
    {"union",      Bool, "Display of 'union' linkage",      &local.display_union},
@@ -352,7 +354,7 @@ static int x_issue_special_command(const char * line, Parse_Options opts, Dictio
 		else
 		{
 			float val = -1.0;
-			val = atof(y);
+			val = (float) atof(y);
 			if (val < 0.0f)
 			{
 				printf("Invalid value %s for variable %s Type \"!help\" or \"!variables\"\n", y, as[j].string);
@@ -421,6 +423,7 @@ static void put_opts_in_local_vars(Parse_Options opts)
 	local.display_disjuncts = parse_options_get_display_disjuncts(opts);
 	local.display_links = parse_options_get_display_links(opts);
 	local.display_senses = parse_options_get_display_senses(opts);
+	local.display_suffixes = parse_options_get_display_suffixes(opts);
 	local.display_walls = parse_options_get_display_walls(opts);
 }
 
@@ -459,6 +462,7 @@ static void put_local_vars_in_opts(Parse_Options opts)
 	parse_options_set_display_disjuncts(opts, local.display_disjuncts);
 	parse_options_set_display_links(opts, local.display_links);
 	parse_options_set_display_senses(opts, local.display_senses);
+	parse_options_set_display_suffixes(opts, local.display_suffixes);
 	parse_options_set_display_walls(opts, local.display_walls);
 }
 
