@@ -174,8 +174,11 @@ static int fget_input_char(FILE *in, FILE *out, Parse_Options opts)
 static void process_linkage(Linkage linkage, Parse_Options opts)
 {
 	char * string;
-	int j, mode, first_sublinkage;
+	int mode;
+#ifdef USE_FAT_LINKAGES
+	int j, first_sublinkage;
 	int nlink;
+#endif /* USE_FAT_LINKAGES */
 
 	if (!linkage) return;  /* Can happen in timeout mode */
 
@@ -187,7 +190,6 @@ static void process_linkage(Linkage linkage, Parse_Options opts)
 		first_sublinkage = linkage_get_num_sublinkages(linkage)-1;
 	}
 	else
-#endif /* USE_FAT_LINKAGES */
 	{
 		first_sublinkage = 0;
 	}
@@ -196,6 +198,7 @@ static void process_linkage(Linkage linkage, Parse_Options opts)
 	for (j=first_sublinkage; j<nlink; ++j)
 	{
 		linkage_set_current_sublinkage(linkage, j);
+#endif /* USE_FAT_LINKAGES */
 		if (parse_options_get_display_on(opts))
 		{
 			string = linkage_print_diagram(linkage);
@@ -226,7 +229,9 @@ static void process_linkage(Linkage linkage, Parse_Options opts)
 			fprintf(stdout, "%s\n", string);
 			linkage_free_postscript(string);
 		}
+#ifdef USE_FAT_LINKAGES
 	}
+#endif /* USE_FAT_LINKAGES */
 	if ((mode = parse_options_get_display_constituents(opts)))
 	{
 		string = linkage_print_constituent_tree(linkage, mode);
@@ -324,20 +329,20 @@ static int process_some_linkages(Sentence sent, Parse_Options opts)
 				fprintf(stdout, "	Linkage %d, ", num_displayed+1);
 			}
 
+#ifdef USE_FAT_LINKAGES
 			if (!linkage_is_canonical(linkage))
 			{
 				fprintf(stdout, "non-canonical, ");
 			}
-#ifdef USE_FAT_LINKAGES
 			if (linkage_is_improper(linkage))
 			{
 				fprintf(stdout, "improper fat linkage, ");
 			}
-#endif /* USE_FAT_LINKAGES */
 			if (linkage_has_inconsistent_domains(linkage))
 			{
 				fprintf(stdout, "inconsistent domains, ");
 			}
+#endif /* USE_FAT_LINKAGES */
 
 			corpus_cost = linkage_corpus_cost(linkage);
 #ifdef USE_FAT_LINKAGES
@@ -424,14 +429,27 @@ static void batch_process_some_linkages(Label label,
                                         Sentence sent,
                                         Parse_Options opts)
 {
-	Linkage linkage;
-
 	if (there_was_an_error(label, sent, opts)) {
-		/* Note: sentence_num_linkages_found returns total linkages
-		 * not valid linkages. So the printed linkage might be bad... 
-		 */
+		/* If linkages were found, prnt them */
 		if (sentence_num_linkages_found(sent) > 0) {
-			linkage = linkage_create(0, sent, opts);
+			Linkage linkage = NULL;
+			/* If we found at least one good linkage, print it. */
+			if (sentence_num_valid_linkages(sent) > 0) {
+				int i;
+				for (i=0; i<sentence_num_linkages_post_processed(sent); i++)
+				{
+					if (0 == sentence_num_violations(sent, i))
+					{
+						linkage = linkage_create(i, sent, opts);
+						break;
+					}
+				}
+			}
+			else
+			{
+				/* This linkage will be bad; no good ones were found. */
+				linkage = linkage_create(0, sent, opts);
+			}
 			process_linkage(linkage, opts);
 			linkage_delete(linkage);
 		}
