@@ -12,6 +12,7 @@
 #include <iostream>
 #include <string>
 #include <assert.h>
+#include <math.h>
 
 #include "atom.h"
 
@@ -24,6 +25,9 @@ const string type_name(AtomType t)
 {
 	switch(t)
 	{
+		case NODE:       return "NODE";
+		// case LINK:       return "LINK";
+
 		case WORD:       return "WORD";
 		case LING_TYPE:  return "LING_TYPE";
 		case CONNECTOR:  return "CONNECTOR";
@@ -34,17 +38,27 @@ const string type_name(AtomType t)
 		case WORD_CSET:  return "WORD_CSET";
 		case WORD_DISJ:  return "WORD_DISJ";
 		case LING:       return "LING";
-		case STATE_PAIR: return "STATE_PAIR";
+		case STATE_TRIPLE: return "STATE_TRIPLE";
 	}
 
 	return "UNHANDLED_TYPE_NAME";
 }
 
+bool TV::operator==(const TV& other) const
+{
+	// The ULP for single-precision floating point is approx 1.0e-7.2
+	if (fabs(other._strength - _strength) < 1.0e-6) return true;
+	return false;
+}
+
+// ====================================================
+
 bool Atom::operator==(const Atom* other) const
 {
 	if (!other) return false;
 	if (other == this) return true;
-	if (other->_type == this->_type) return true;
+	if (other->_type == this->_type and
+	    other->_tv == this->_tv) return true;
 	return false;
 }
 
@@ -53,6 +67,7 @@ bool Node::operator==(const Atom* other) const
 	if (!other) return false;
 	if (other == this) return true;
 	if (other->get_type() != this->get_type()) return false;
+	if (not other->_tv.operator==(this->_tv)) return false;
 	const Node* nother = dynamic_cast<const Node*>(other);
 	if (nother->_name != this->_name)  return false;
 	return true;
@@ -63,6 +78,7 @@ bool Link::operator==(const Atom* other) const
 	if (!other) return false;
 	if (other == this) return true;
 	if (other->get_type() != this->get_type()) return false;
+	if (not other->_tv.operator==(this->_tv)) return false;
 	const Link* lother = dynamic_cast<const Link*>(other);
 	if (lother->get_arity() != this->get_arity())  return false;
 	for (size_t i=0; i<get_arity(); i++)
@@ -80,15 +96,21 @@ std::ostream& do_prt(std::ostream& out, const Atom* a, int ilvl)
 	if (n)
 	{
 		for (int i=0; i<ilvl; i++) cout << indent_str;
-		out << type_name(n->get_type()) <<" : ";
-		out << n->get_name() << endl;
+		out << type_name(n->get_type()) << " : "
+		    << n->get_name();
+		if (n->_tv._strength > 0.0f)
+			out << "    (" << n->_tv._strength << ")";
+		out << endl;
 		return out;
 	}
 	const Link* l = dynamic_cast<const Link*>(a);
 	if (l)
 	{
 		for (int i=0; i<ilvl; i++) cout << indent_str;
-		out << type_name(l->get_type()) <<" :" << endl;
+		out << type_name(l->get_type()) <<" :";
+		if (l->_tv._strength > 0.0f)
+			out << "     (" << l->_tv._strength << ")";
+      out << endl;
 
 		ilvl++;
 		size_t lsz = l->get_arity();
