@@ -22,18 +22,15 @@ namespace viterbi {
 class Connect : public gc
 {
 	public:
-		Connect(WordCset*);
-		Set* try_connect(StatePair*);
+		Connect(WordCset*, WordCset*);
+		StateTriple* try_alternative(Atom*, Atom*);
 
 	protected:
-		Set* next_connect(WordCset*);
 
-		StatePair* try_alternative(Atom*, Atom*);
-
-		StatePair* alternative(Connector*, Connector*);
-		StatePair* alternative(Connector*, And*);
-		StatePair* alternative(And*, Connector*);
-		StatePair* alternative(And*, And*);
+		StateTriple* alternative(Connector*, Connector*);
+		StateTriple* alternative(Connector*, And*);
+		StateTriple* alternative(And*, Connector*);
+		StateTriple* alternative(And*, And*);
 
 		Ling* conn_connect_nn(Connector*, Connector*);
 		Ling* reassemble(Ling*, WordCset*, WordCset*);
@@ -42,7 +39,6 @@ class Connect : public gc
 
 	private:
 		WordCset* _right_cset;
-		Atom* _rcons;  // just the connector(s), without the word.
 		WordCset* _left_cset;
 };
 
